#if !defined(CSV_PARSER_H_INCLUDED_)
#define CSV_PARSER_H_INCLUDED_

#pragma once

#include <istream>
#include <fstream>
#include <sstream>
#include <vector>

using WStringCollection		= std::vector<std::wstring>;
using WStringCollectionR	= WStringCollection&;
using WStringCollectionCR	= WStringCollection const&;

///	<summary>A class to simplify Comma-Separated-Variable (CSV) text file parsing.</summary>
class CSVRow
{
	const wchar_t		Separator_;
	///	<summary>A collection of words parsed from a row from a CSV file.</summary>
	WStringCollection   data_;

public:
	CSVRow	() : Separator_ (L',') {}
	///	<summary>Get the word from our row at column (index).</summary>
	std::wstring const& operator[](std::size_t index) const
	{
		return data_ [index];
	}
	///	<summary>How many words (columns) in the current row.</summary>
	std::size_t size () const
	{
		return data_.size();
	}
	///	<summary>Parse a row from the CSV input stream.</summary>
	void ReadNextRow (std::wistream& str);
};
///	<summary>Read from an input stream into a CSV parser.</summary>
std::wistream& operator>>(std::wistream& str, CSVRow& data);

#endif // !defined(CSV_PARSER_H_INCLUDED_)
