#include <Mstn/MdlApi/mdl.h>
#include <DgnPlatform/DgnPlatform.h>
#include <Bentley/BeFileName.h>
#include <Mstn/MdlApi/dlogitem.h>
#include <Mstn/MdlApi/ditemlib.fdf>
#include <Mstn/MdlApi/msdialog.h>
#include <Mstn/MdlApi/msdialog.fdf>
#include <Mstn/MdlApi/leveltable.fdf>
#include <Mstn/MdlApi/msmisc.fdf>
#include <Mstn/MdlApi/miscilib.fdf>
#include <Mstn/MdlApi/msritem.fdf>
#include <Mstn/MdlApi/mssystem.fdf>
#include <Mstn/MdlApi/mswindow.fdf>
#include <Mstn/MdlApi/workmode.fdf>
#include "datasheet.h"
#include "HookClasses.h"
#include "MessageTracer.h"

extern GlobalData  		gData;

const bool		EditDisplay			{ false };
const bool		UpdateDisplay		{ true };
const bool		SetDisplayString	{ true };
///	<summary>Get a pointer to the DataSheet ListModel in from the parent dialog box.</summary>
///	<remarks>Some cell editors, such as a ColorPicker, are hosted by a Container.
///	Do not user the container's pointer in error.</remarks>
ListModelP	GetDataListModel (MSDialogP	db)
{
	DialogItemP		lBox 		= mdlDialog_itemGetByTypeAndId (db, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListModelP		pDataList	= mdlDialog_listBoxGetListModelP (lBox->rawItemP);
	return pDataList;
}
///////////////////////////////////////////////////////////////////////
//	static
ColourDescriptors DataListBoxHandler::colours_;
///////////////////////////////////////////////////////////////////////
///	<summary>When user has finished with a ListModel cell editor in a container, invoke this common action.</summary>
void	DataListBoxHandler::CloseEditor (MSDialogP	db)
{
	mdlDialog_lastActionTypeSet (db, ACTIONBUTTON_OK);
	mdlWindow_hide ((MSWindowP)db, HideReasonCode::HideWindow, true);
	mdlDialog_closeCommandQueue (db);
}
///////////////////////////////////////////////////////////////////////
//	static
ListCellP			DataListBoxHandler::GetEditCell
(
MSDialogP		 	db,			//	=>	Dialog that contains ComboBox or ListBox
RawItemHdr*			rItem		//	=>	Reference to  ComboBox or ListBox
)
{
	ListCellP		pCell		= nullptr;	//	This is the return value
	DialogItemP		pContainer	= rItem->diP;

	switch (pContainer->type)
	{
		case RTYPE_ListBox:
		{
			ListModelP	pListModel	= mdlDialog_listBoxGetListModelP (rItem);
			int			row = 0;
			int			col = 0;
			int			status		= mdlDialog_listBoxGetEditCell (&row, &col, rItem);
			if (SUCCESS == status)
			{
				pCell	= mdlListModel_getCellAtIndexes (pListModel, row, col);
				//CMessageTracer msg (L"mdlListModel_getCellAtIndexes (row="); msg += row; msg += ","; msg += col; msg += ")";
				//msg += L" pCell is "; msg += nullptr == pCell? L"nullptr" : L"valid";
				//msg.Output ();
			}
			else
			{
				CMessageTracer moan (L"mdlDialog_listBoxGetEditCell (RTYPE_ListBox) failed");
				moan.Advisory ();
			}
			break;
		}
		case RTYPE_ComboBox:
		{
			long		row = 0;
			ListModelP	pComboList	= mdlDialog_comboBoxGetListModelP (rItem);
			int			status		= mdlDialog_comboBoxGetIndex (&row, rItem);
			if (SUCCESS == status)
			{
				ListRowP 	pRow	= mdlListModel_getRowAtIndex (pComboList, row);
				pCell	= mdlListRow_getCellAtIndex (pRow, 0);
				//CMessageTracer msg (L"mdlDialog_comboBoxGetIndex (row="); msg += row;
				//msg += L" pCell is "; msg += nullptr == pCell? L"nullptr" : L"valid";
				//msg.Output ();
			}
			else
			{
				CMessageTracer moan (L"mdlDialog_comboBoxGetIndex (RTYPE_ComboBox) failed");
				moan.Advisory ();
			}
			break;
		}
	}
	return pCell;
}
///////////////////////////////////////////////////////////////////////
ListModelP			DataListBoxHandler::CreateListModel
(
int					nCols
)
{
	ListModelP		pListModel = mdlListModel_create (nCols);
	const int		nRows		= 6;	//	An arbitrary number of rows
	for (int rowIndex = 0; rowIndex != nRows; ++rowIndex)
	{
		ListRowP	pRow = mdlListRow_create (pListModel);
		for (int colIndex = 0; colIndex < nCols; ++colIndex)
		{
			WChar		value 	[32];
			swprintf (value, L"value %d.%d", rowIndex, colIndex);
			ListCellP	pCell = mdlListRow_getCellAtIndex (pRow, colIndex);

			switch (colIndex)
			{
			case colSimpleTextEditor:
				//	Column 1 shows plain text with text editor
				mdlListCell_setStringValue (pCell, value, SetDisplayString);
				mdlListCell_setEditor (pCell, RTYPE_Text, TEXTID_UserInfo,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				//mdlListCell_setEditor (pCell, RTYPE_Container, CONTAINERID_TextEditor, mdlSystem_getCurrMdlDesc (), FALSE, TRUE);
				break;
			case colComboTextEditor:
				//	Column 2 shows text provided by a ComboBox editor
				mdlListCell_setStringValue (pCell, value, SetDisplayString);
				mdlListCell_setEditor (pCell, RTYPE_ComboBox, COMBOBOXID_DataSheetEditor,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			case colRowCombo:
				//	Column 6 shows text provided by a ComboBox editor.  The ComboBox
				//	content is unique for each row
				mdlListCell_setStringValue (pCell, value, SetDisplayString);
				mdlListCell_setEditor (pCell, RTYPE_ComboBox, COMBOBOXID_PerRow,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			case colLevelCombo:
			{
				//	Column 5 displays level edited using a ComboBox
				mdlListCell_setStringValue (pCell, value, SetDisplayString);
				//	COMBOBOXID_Levels uses HOOKITEMID_ComboBoxLevels
				mdlListCell_setEditor (pCell, RTYPE_ComboBox, COMBOBOXID_Levels,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			}
			case colComboCheckBox:
			{
				//	Column 3 displays a cross or tick icon and is edited
				//	either using a ComboBox to display Yes or No
				//	or by a double-click handled by the ListBox hook function
				const long	flag = (long)(rowIndex % 2 ? 1 : 0);
				mdlListCell_setLongValue (pCell, flag);
				DataListBoxHandler::SetListCellIconByInternalValue (pCell);

				mdlListCell_setEditor (pCell, RTYPE_ComboBox, COMBOBOXID_DataSheetYesNo,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			}
			case colColourSwatch:
			{
				//	Column 4 displays a Color patch and is edited
				//	using a ColorChooser in a Container
				//	or a text item
				const long 	color = (rowIndex * nCols + colIndex) % 256;
				mdlListCell_setLongValue (pCell, color);
				mdlListCell_setIcon (pCell, ICONID_ColorPatch, RTYPE_Icon, mdlSystem_getCurrMdlDesc ());
				DataListBoxHandler::Colours ().SetForegroundColour (color);
				DataListBoxHandler::Colours ().SetIconColorOverrides (pCell);

				//	Text widget works OK for editing numeric Color palette index
				//mdlListCell_setEditor (pCell, RTYPE_Text, TEXTID_Color,
				//		mdlSystem_getCurrMdlDesc (), FALSE, TRUE);
				//mdlListCell_setEditor (pCell, RTYPE_ColorPicker, COLORPICKERID_DataSheetClr,
				//		mdlSystem_getCurrMdlDesc (), FALSE, TRUE);
				//  CONTAINERID_ColorChooser contains COLORCHOOSERID_Color
				mdlListCell_setEditor (pCell, RTYPE_Container, CONTAINERID_ColorChooser, mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			}
			}
		}
		mdlListModel_addRow (pListModel, pRow);
	}

	return pListModel;
}
///////////////////////////////////////////////////////////////////////
bool	DataListBoxHandler::_OnCreate			(DialogItemCreateArgsR 			create)
{
	ListModelP	pListModel = CreateListModel (nColumns);
	mdlDialog_listBoxSetListModelP (GetRawItem (), pListModel, nColumns);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool	DataListBoxHandler::_OnDestroy			()
{
	ListModelP	pListModel = mdlDialog_listBoxGetListModelP (GetRawItem ());
	if (pListModel)
		mdlListModel_destroy (pListModel, true);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool	DataListBoxHandler::_OnButton			(DialogItemButtonArgsR 			button)
{
	if  (BUTTONTRANS_UP == button.buttonTrans)
	{
		int	row, col;
		if (SUCCESS == mdlDialog_listBoxLastCellClicked (&row, &col, GetRawItem ()))
		{
			mdlDialog_listBoxEditCell (GetRawItem (), row, col);
		}
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
//	static
bool				DataListBoxHandler::SetListCellIconByInternalValue
(
ListCellP			pCell				//	=>	Modify this cell icon
)
{
	long			flag	= -1;
	bool			rc		= false;
	if (SUCCESS == mdlListCell_getLongValue (pCell, &flag))
	{
		mdlListCell_setIcon (pCell, flag ? ICONID_ColorCheckMark : 0, RTYPE_Icon, mdlSystem_getCurrMdlDesc ());
		//mdlListCell_setIcon (pCell, flag ? ICONID_SmallCheck : ICONID_SmallCross, RTYPE_Icon, mdlSystem_getCurrMdlDesc ());
		rc = true;
	}
	return rc;
}
///////////////////////////////////////////////////////////////////////
bool 	ListCellTextEditor::_OnStateChanged	(DialogItemStateChangedArgsR	stateChanged)
{
	if  (stateChanged.reallyChanged )
	{
		//	Get the value of this text editor
		MSValueDescr	value;
		mdlDialog_itemGetValue (value, GetDialog (), GetItemIndex ());
		//	Apply the value in the DataSheet ListModel
		ListModelP	pDataList	= GetDataListModel (GetDialog ());
		int    		row, col;
		DialogItemP	lBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
		mdlDialog_listBoxGetEditCell (&row, &col, lBox->rawItemP);
		ListCellP	pCell = mdlListModel_getCellAtIndexes (pDataList, row, col);
		mdlListCell_setStringValue (pCell, value.GetWCharCP (), SetDisplayString);
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ListCellTextEditor::_OnInit			(DialogItemInitArgsR 			init)
{
	//get the value in the listbox then transfer it to the text box.
	//this should be unnecessary for the simple case since a text item is "created"
	//behind the scenes.
	ListModelP		pDataList	= GetDataListModel (GetDialog ());
	int             row, col;
	mdlDialog_listBoxGetEditCell (&row, &col, GetRawItem ());
	ListCellP		pCell	= mdlListModel_getCellAtIndexes (pDataList, row, col);
	MSValueDescr	value;
	mdlListCell_getValue (pCell, value);
	//	Set the string in the text item.
	mdlDialog_rItemValueSet (nullptr, value, GetRawItem ());
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_TextEditor::_OnCreate			(DialogItemCreateArgsR 			create)
{
	const int	nColumns	= 1;
	ListModelP	pListModel	= mdlListModel_create (nColumns);
	for (int x = 0; x <= 10 ; ++x)
	{
		WChar    	xs[4];
		ListRowP	pRow	= mdlListRow_create (pListModel);
		ListCellP	pCell	= mdlListRow_getCellAtIndex (pRow, 0);
		swprintf (xs, L"x%ld", x);
		mdlListCell_setStringValue (pCell, xs, true);
		mdlListModel_addRow (pListModel, pRow);
	}
	mdlDialog_comboBoxSetListModelP (GetRawItem (), pListModel);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_TextEditor::_OnDestroy			()
{
	ListModelP	pListModel = mdlDialog_comboBoxGetListModelP (GetRawItem ());
	if (pListModel)
		mdlListModel_destroy (pListModel, true);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_TextEditor::_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)
{
	WString		string;
	//	Get the value of the current ListCell from this ComboBox
	ListCellP	pComboCell = DataListBoxHandler::GetEditCell (GetDialog (), GetRawItem ());
	mdlListCell_getStringValueWString  (string, pComboCell);
	//	Assign the value to the current ListCell in the DataSheet ListBox
	DialogItemP	pLBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP	pDataCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	mdlListCell_setStringValue (pDataCell, string.c_str (), SetDisplayString);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_BoolEditor::_OnCreate			(DialogItemCreateArgsR 			init)
{
	//	This is the ComboBox internal ListModel
	//	Add one row for Yes
	const int	nColumns	= 1;
	ListModelP	pComboList	= mdlListModel_create (nColumns);
	ListRowP	pRow 		= mdlListRow_create (pComboList);
	ListCellP	pCell 		= mdlListRow_getCellAtIndex (pRow, 0);
	mdlListCell_setStringValue (pCell, L"Yes", true);
	mdlListModel_addRow (pComboList, pRow);
	//	Add one row for No
	pRow	= mdlListRow_create (pComboList);
	pCell	= mdlListRow_getCellAtIndex (pRow, 0);
	mdlListCell_setStringValue (pCell, L"No", true);
	mdlListModel_addRow (pComboList, pRow);
	mdlDialog_comboBoxSetListModelP (GetRawItem (), pComboList);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_BoolEditor::_OnDestroy			()
{
	ListModelP	pListModel = mdlDialog_comboBoxGetListModelP (GetRawItem ());
	if (pListModel)
		mdlListModel_destroy (pListModel, true);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_BoolEditor::_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)
{
	long     	yesNo	= -1;
	mdlDialog_comboBoxGetIndex (&yesNo, GetRawItem ());
	//	Get the corresponding cell in the DataSheet ListBox
	DialogItemP	pLBox	= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP	pCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	if (pCell)
	{
		long		value		= (long)(0 == yesNo ? 1: 0);
		mdlListCell_setLongValue (pCell, value);
		DataListBoxHandler::SetListCellIconByInternalValue (pCell);
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
ListModelP			ComboBoxCell_LevelEditor::BuildLevelList
(
)
{
	CMessageTracer	msg (L"ComboBoxCell_LevelEditor::BuildLevelList");
	msg.Output ();

	const int		nColumns	= 1;
	ListModelP		pListModel	= mdlListModel_create (nColumns);
	//	mdlLevelList_getLevelNames creates a ListModel that stores
	//	the level ID in each ListCell
	DgnPlatform::LevelId	levelId;
	ActiveParams::GetValue (levelId, ACTIVEPARAM_LEVEL);
	if (SUCCESS == mdlLevelList_getLevelNamesListModel (pListModel, ACTIVEMODEL, levelId, LEVELLIST_BOLD_USED ))
	{
		for (int row = 0; row != mdlListModel_getRowCount  (pListModel); ++row)
		{
			ListCellP		pCell  = mdlListModel_getCellAtIndexes (pListModel, row, 0);
			WCharCP			display;
			mdlListCell_getDisplayText (pCell, &display);
			MSValueDescr	value;
			mdlListCell_getValue (pCell, value);
			msg = "ListRow "; msg [row];
			msg += " Display"; msg.Quote (display);
			FormatType	fmt	= value.GetFormatType ();
			msg += " Internal formattype="; msg += fmt; msg.Quote ( FMT_UINT == fmt? "Unsigned Int": "other");
			msg.Output ();
		}
	}
	else
	{
		mdlListModel_destroy (pListModel, true);
	}

	return pListModel;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_LevelEditor::_OnCreate			(DialogItemCreateArgsR 			init)
{
	int				i			= -1;
	//	This is this ComboBox's internal ListModel
	ListModelP		pComboList = BuildLevelList ();
	mdlDialog_comboBoxSetListModelP (GetRawItem (), pComboList);
	//	Synchronise combo display with current ListCell contents from DataSheet
	DialogItemP		pListBox	= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListModelP 		pDataList	= mdlDialog_listBoxGetListModelP (pListBox->rawItemP);
	int				row = -1, col = -1;
	//	Get the DataSheet ListBox current cell
	mdlDialog_listBoxGetEditCell (&row, &col, pListBox->rawItemP );
	ListCellP		pCell = mdlListModel_getCellAtIndexes (pDataList, row, col);
	WString			level;
	mdlListCell_getDisplayTextWString (pCell, level);
	if (!level.empty ())
	{	//	Attempt to match a row in this ComboBox
		for (i = 0; i != mdlListModel_getRowCount (pComboList); ++i)
		{
			pCell	= mdlListModel_getCellAtIndexes (pComboList, i, 0);
			MSValueDescr	value;
			mdlListCell_getValue (pCell, value);
			WChar			wcLevel		[Bentley::DgnPlatform::MAX_LEVEL_NAME_LENGTH];
			mdlLevel_getName  (wcLevel, Bentley::DgnPlatform::MAX_LEVEL_NAME_LENGTH, ACTIVEMODEL, value.GetLong ());
			if (0 == wcsicmp (level.c_str (), wcLevel))
			{
				mdlDialog_comboBoxSetIndex (i, GetRawItem ());
				break;
			}
		}
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_LevelEditor::_OnDestroy			()
{
	ListModelP	pListModel = mdlDialog_comboBoxGetListModelP (GetRawItem ());
	if (pListModel)
		mdlListModel_destroy (pListModel, true);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_LevelEditor::_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)
{
	//	Get user selection from ComboBox ListModel
	long		nRow	= -1;
	mdlDialog_comboBoxGetIndex (&nRow, GetRawItem ());
	ListModelP	pComboList	= mdlDialog_comboBoxGetListModelP (GetRawItem ());
	ListRowP	pRow		= mdlListModel_getRowAtIndex (pComboList, nRow);
	ListCellP	pCell 		= mdlListRow_getCellAtIndex (pRow, 0);
	WString 	levelName;
	mdlListCell_getDisplayTextWString (pCell, levelName);
	//	Apply value to DataSheet ListModel
	DialogItemP	pListBox	= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListModelP 	pDataList	= mdlDialog_listBoxGetListModelP (pListBox->rawItemP);
	int			row = -1, col = -1;
	mdlDialog_listBoxGetEditCell (&row, &col, pListBox->rawItemP );
	pCell = mdlListModel_getCellAtIndexes (pDataList, row, col);
	if (pCell)
	{
		 mdlListCell_setDisplayText (pCell, levelName.c_str ());
		//mdlListCell_setStringValue (pCell, levelName.c_str (), true);
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ToggleButtonCell_BoolEditor::_OnCreate			(DialogItemCreateArgsR 			create)
{
	bool				changed		{ false };
	DialogItemP			pLBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP			pDataCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	if (pDataCell)
	{
		long			val;
		mdlListCell_getLongValue (pDataCell, &val);
		MSValueDescr	value (val);
		mdlDialog_itemSetValue (&changed, value, GetDialog (), GetItemIndex ());
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ToggleButtonCell_BoolEditor::_OnButton			(DialogItemButtonArgsR 			button)
{
		if ((BUTTONTRANS_UP == button.buttonTrans)
			&&
			(DATAPNT == button.buttonNumber)
			&&
			(button.clicked))
		{
			//msg ="ToggleButton BUTTON clicked ", msg += (1 == dimP->u.button.upNumber)? "once": "twice";
			//msg.Output ();
		}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ListCellColorEditor::_OnCreate			(DialogItemCreateArgsR 			init)
{
	long			initialise { -1 };
	DialogItemP			pLBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP			pDataCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	if (pDataCell)
	{
		mdlListCell_getLongValue (pDataCell, &initialise);
		WChar			value [32];
		swprintf (value, L"%d", (int)initialise);
		bool			changed	= false;
		mdlDialog_itemSetStringValue (&changed, value, GetDialog (), GetItemIndex ());
		mdlDialog_itemSetState (&changed, GetDialog (), GetItemIndex ());
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool  	ListCellColorEditor::_OnQueueCommand 	(DialogItemQueueCommandArgsR 	queueCommand)
{
	MSValueDescr		value;
	mdlDialog_itemGetValue (value, GetDialog (), GetItemIndex ());
	const int			color { _wtoi (value.GetWCharCP ()) };
	DialogItemP			pLBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP			pDataCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	if	(pDataCell
		&&
		(SUCCESS == mdlListCell_setLongValue (pDataCell, _wtol (value.GetWCharCP ()))))
	{
		DataListBoxHandler::Colours ().SetForegroundColour (color);
		mdlListCell_setIconColorOverrides (pDataCell, DataListBoxHandler::Colours ().GetBackgroundColour (), DataListBoxHandler::Colours ().GetForegroundColour ());
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ColorPickerCell_ColorEditor::_OnCreate			(DialogItemCreateArgsR 			init)
{
	DialogItemP			pLBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP			pDataCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	if (pDataCell)
	{
		//	Get value from DataSheet ListModel
		long			lVal = 0;
		mdlListCell_getLongValue (pDataCell, &lVal);
		MSValueDescr	value (lVal);
		bool			changed	= false;
		//	Apply value to this dialog item
		mdlDialog_itemSetValue (&changed, value, GetDialog (), GetItemIndex ());
		mdlDialog_itemSetState (&changed, GetDialog (), GetItemIndex ());
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ColorPickerCell_ColorEditor::_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)
{
	MSValueDescr		value;
	mdlDialog_itemGetValue (value, GetDialog (), GetItemIndex ());
	int					color 		= (int)value.GetLong ();
	DialogItemP			pLBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP			pDataCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	if (pDataCell)
	{
		DataListBoxHandler::Colours ().SetForegroundColour (color);
		mdlListCell_setIconColorOverrides (pDataCell, DataListBoxHandler::Colours ().GetBackgroundColour (), DataListBoxHandler::Colours ().GetForegroundColour ());
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ColorChooserCell_ColorEditor::_OnCreate			(DialogItemCreateArgsR 			create)
{
	int		    	row = -1, col = -1;
	//	The dialog that owns this widget is a Container,
	//	which is not the same as the Dialog that contains the ListBox
	MSDialogP		dbP		= mdlDialog_find (DIALOGID_SpreadSheet, 0);
	DialogItemP		pLBox	= mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	mdlDialog_listBoxGetEditCell (&row, &col, pLBox->rawItemP);
	ListCellP		pDataCell	= DataListBoxHandler::GetEditCell (dbP, pLBox->rawItemP);
	if (nullptr == pDataCell)
	{
		CMessageTracer moan (L"ColorChooser_CREATE  pCell == NULL dialogID="); moan += GetDialog ()->GetResourceId ();
		moan.Advisory ();
	}
	else
	{
		long			lVal;
		mdlListCell_getLongValue (pDataCell, &lVal);
		bool			changed;
		MSValueDescr	value (lVal);
		mdlDialog_itemSetValue (&changed, value, GetDialog (), GetItemIndex ());
	}
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ColorChooserCell_ColorEditor::_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)
{
	///	<summary>Copy the colour index from the ColorChooser to the DataSheet ListModel and set its colour swatch.</summary>
	///	<remarks>	The dialog that owns this ColorChooser is a Container.
	///	The pointer dimP->db references the Container,
	///	which is not the same as the Dialog that contains the ListBox.</remarks>
	MSDialogP		dbP			= mdlDialog_find (DIALOGID_SpreadSheet, NULL);
	DialogItem*		pLBox		= mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListModelP		pModel		= mdlDialog_listBoxGetListModelP (pLBox->rawItemP);
	//	Get a reference to the ListBox current ListCell
	int		    	row 		= -1, col = -1;
	mdlDialog_listBoxGetEditCell (&row, &col, pLBox->rawItemP);
	ListRowP		pRow		= mdlListModel_getRowAtIndex (pModel, row);
	ListCellP		pDataCell	= mdlListRow_getCellAtIndex (pRow, col);
	//	Get this ColorChooser's Color index
	MSValueDescr	value;
	mdlDialog_itemGetValue (value, GetDialog (), GetItemIndex ());
	int				colour		{ (int)value.GetLong () };
	if (0 <= colour && colour < 256)
	{
		mdlListCell_setLongValue (pDataCell, colour);
		DataListBoxHandler::Colours ().SetForegroundColour (colour);
		DataListBoxHandler::Colours ().SetIconColorOverrides (pDataCell);
		mdlDialog_itemDraw (dbP, pLBox->itemIndex);
	}
	//	Inform the ListCell that we've finished editing.  This is required because
	//	this dialog item is hosted by a Container.
	DataListBoxHandler::CloseEditor (GetDialog ());
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_PerRow::_OnCreate			(DialogItemCreateArgsR 			init)
{

	MSDialogP		dbP			= mdlDialog_find (DIALOGID_SpreadSheet, NULL);
	DialogItem*		pLBox		= mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	//ListModelP		pDataList	= mdlDialog_listBoxGetListModelP (pLBox->rawItemP);
	//	Get a reference to the ListBox current ListCell
	int		    	row 		= -1, col = -1;
	mdlDialog_listBoxGetEditCell (&row, &col, pLBox->rawItemP);
	//ListRowP		pRow		= mdlListModel_getRowAtIndex (pDataList, row);
	//ListCellP		pDataCell	= mdlListRow_getCellAtIndex (pRow, col);
	WChar			s [32];
	ListModelP		comboList	= mdlListModel_create (1);
	for (int i = 0; i != 6; ++i)
	{
		ListRowP	pRow = mdlListRow_create (comboList);
		ListCellP	pCell = mdlListRow_getCellAtIndex (pRow, 0);
		swprintf (s, L"value %d.%d [%d]", col, i, row);
		mdlListCell_setStringValue (pCell, s, SetDisplayString);
		mdlListModel_addRow (comboList, pRow);
	}
	mdlDialog_comboBoxSetListModelP (GetRawItem (), comboList);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_PerRow::_OnDestroy			()
{
	ListModelP	comboList = mdlDialog_comboBoxGetListModelP (GetRawItem ());
	if (comboList)
		mdlListModel_destroy (comboList, true);
	return true;
}
///////////////////////////////////////////////////////////////////////
bool 	ComboBoxCell_PerRow::_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)
{
	WString		string;
	//	Get the value of the current ListCell from this ComboBox
	ListCellP	pComboCell = DataListBoxHandler::GetEditCell (GetDialog (), GetRawItem ());
	mdlListCell_getStringValueWString  (string, pComboCell);
	//	Assign the value to the current ListCell in the DataSheet ListBox
	DialogItemP	pLBox		= mdlDialog_itemGetByTypeAndId (GetDialog (), RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	ListCellP	pDataCell	= DataListBoxHandler::GetEditCell (GetDialog (), pLBox->rawItemP);
	mdlListCell_setStringValue (pDataCell, string.c_str (), SetDisplayString);
	return true;
}
///////////////////////////////////////////////////////////////////////
