#if !defined(HOOK_CLASSES_H_INCLUDED_)
#define HOOK_CLASSES_H_INCLUDED_

#pragma once

struct	ColourDescriptors
{
	BSIColorDescr*	pBackgroundColor_;
	BSIColorDescr*	pForegroundColor_;

	void			SetForegroundColour (BSIColorDescr*	color) {	pForegroundColor_	= color;	}
	void			SetForegroundColour (long 			color) {	pForegroundColor_ 	= mdlColorPal_getColorDescr (nullptr, color);	}
	void			SetBackgroundColour (BSIColorDescr*	color) {	pBackgroundColor_	= color;	}
	void			SetBackgroundColour (long 			color) {	pBackgroundColor_	= mdlColorPal_getColorDescr (nullptr, color);	}
	BSIColorDescr*	GetForegroundColour	()	const	{	return	pForegroundColor_;	}
	BSIColorDescr*	GetBackgroundColour	()	const 	{	return	pBackgroundColor_;	}

	void			SetIconColorOverrides (ListCellP	pCell) { mdlListCell_setIconColorOverrides (pCell, pBackgroundColor_, pForegroundColor_);	}
};

using ColourDescriptorsR	= ColourDescriptors&;
using ColourDescriptorsCR	= ColourDescriptors const&;

///////////////////////////////////////////////////////////////////////
///	<summary>Populates the DataSheet ListBox and handles button click events.</summary>
///	<remarks>This handler is around for the lifetime of the ListBox hosted by the
///	DataSheet dialog.  In contrast, some of the dialog item editing handlers are
/// created on-demand and expire once the edit is finished.
///	</remarks>
struct DataListBoxHandler : DialogItemHookHandler
{
	///	<remarks>Colour descriptors are owned by this DataListBoxHandler and
	///	are used by other classes as well as this one.</remarks>
	static	ColourDescriptors	colours_;
	static 	ColourDescriptorsR	Colours	()			{	return colours_; 	}

	enum ColumnIndices
	{
		colSimpleTextEditor	= 0,
		colComboTextEditor,
		colComboCheckBox,
		colColourSwatch,
		colLevelCombo,
		colRowCombo,
		nColumns,
	};
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new DataListBoxHandler (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	DataListBoxHandler (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	///	<summary> Item Hook Handler Method overrides
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 		override;
	virtual bool 	_OnDestroy			() 		override;
	virtual bool 	_OnButton			(DialogItemButtonArgsR 			init) 		override;
	ListModelP		CreateListModel		(int							nCols);
	///	<summary>
	///	A ListCell is contained by a ListModel that is
	///	contained by a ListBox or ComboBox.  The ListModel
	///	contains an array of ListCell objects.	A reference
	///	to the currently-selected ListCell is often
	///	required to get or set the ListCell's contents.
	///	</summary>
	///	<remarks>
	///	Don't call from inside a ListBox hook function.
	///	Instead, get the current row and column from the
	///	ListBox:
	///	mdlDialog_listBoxLastCellClicked ()
	///	Next, get the associated ListModel:
	///	mdlDialog_listBoxGetListModelP ()
	///	Next, get the ListCell at row, colum:
	///	mdlListModel_getCellAtIndexes ()
	///	</remarks>
	static ListCellP	GetEditCell						(MSDialogP		 	db,
														RawItemHdr*			rItem);
	///	<summary>
	///	Assuming the long data member of a ListCell holds
	///	a binary value, set its icon to check/blank or
	///	some other visual representation of binary state
	///	</summary>
	static	bool		SetListCellIconByInternalValue	(ListCellP			pCell);
	///	<summary>When user has finished with a ListModel cell editor in a container, invoke this common action.</summary>
	static	void		CloseEditor 					(MSDialogP			db);
};
///////////////////////////////////////////////////////////////////////
///	<summary>ComboBox used to Yes/No response ListBox cell.</summary>
///	<remarks>The Boolean state is used to specify a ListCell icon (tick mark for Yes, blank for No).</remarks>
struct ComboBoxCell_BoolEditor : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ComboBoxCell_BoolEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ComboBoxCell_BoolEditor (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 		override;
	virtual bool 	_OnDestroy			() 		override;
    virtual bool 	_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)	override;
};
///////////////////////////////////////////////////////////////////////
///	<summary>ComboBox has row-specific content that allows a user to pick a text value to transfer to a ListBox cell.</summary>
struct ComboBoxCell_PerRow  : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ComboBoxCell_PerRow (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ComboBoxCell_PerRow (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 		override;
	virtual bool 	_OnDestroy			() 		override;
    virtual bool 	_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)	override;
};
///////////////////////////////////////////////////////////////////////
///	<summary>ComboBox used to pick a text value to transfer to a ListBox cell.</summary>
struct ComboBoxCell_TextEditor  : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ComboBoxCell_TextEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ComboBoxCell_TextEditor (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 		override;
	virtual bool 	_OnDestroy			() 		override;
    virtual bool 	_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)	override;
};
///////////////////////////////////////////////////////////////////////
///	<summary>ComboBox used to pick a level name to transfer to a ListBox cell.</summary>
struct ComboBoxCell_LevelEditor  : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ComboBoxCell_LevelEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ComboBoxCell_LevelEditor (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 		override;
	virtual bool 	_OnDestroy			() 		override;
    virtual bool 	_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)	override;
    ListModelP		BuildLevelList		();
};
///////////////////////////////////////////////////////////////////////
///	<summary>A simple text editor for a ListCell.</summary>
struct ListCellTextEditor : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ListCellTextEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ListCellTextEditor (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	///	<summary> Item Hook Handler Method overrides
	virtual bool 	_OnInit			(DialogItemInitArgsR 			init) 		override;
    virtual bool 	_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)	override;
	// Other hook handler method overrides...
};
///////////////////////////////////////////////////////////////////////
///	<summary>A simple editor that applies colour to a ListCell.</summary>
///	<remarks>Prototype: there is no corresponding item in the DataSheet example.</remarks>
struct ListCellColorEditor : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ListCellColorEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ListCellColorEditor (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	///	<summary> Item Hook Handler Method overrides
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 			override;
	virtual bool  	_OnQueueCommand 	(DialogItemQueueCommandArgsR 	queueCommand)	override;
	// Other hook handler method overrides...
};
///////////////////////////////////////////////////////////////////////
///	<summary>A simple Toggle Button text editor for a ListCell.</summary>
///	<remarks>Prototype: there is no Toggle Button in the DataSheet example.</remarks>
struct ToggleButtonCell_BoolEditor : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ToggleButtonCell_BoolEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ToggleButtonCell_BoolEditor (MSDialogP dbP, DialogItemP diP) : DialogItemHookHandler (dbP, diP) {}
	///	<summary> Item Hook Handler Method overrides
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 		override;
	virtual bool 	_OnButton			(DialogItemButtonArgsR 			init) 		override;
	// Other hook handler method overrides...
};
///////////////////////////////////////////////////////////////////////
///	<summary>Pop a ColorPicker item to change the colour swatch displayed by a ListCell.</summary>
///	<remarks>Prototype: there is no ColorPicker in the DataSheet example.</remarks>
struct ColorPickerCell_ColorEditor : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ColorPickerCell_ColorEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ColorPickerCell_ColorEditor (MSDialogP dbP, DialogItemP diP) :
		DialogItemHookHandler (dbP, diP) {}
	///	<summary> Item Hook Handler Method overrides
	virtual bool 	_OnCreate		(DialogItemCreateArgsR 			init) 		override;
    virtual bool 	_OnStateChanged	(DialogItemStateChangedArgsR 	stateChanged)	override;
	// Other hook handler method overrides...
};
///////////////////////////////////////////////////////////////////////
///	<summary>Pop a ColorChooser item to change the colour swatch displayed by a ListCell.</summary>
///	<remarks>The ColorChooser is hosted by a Container.</remarks>
struct ColorChooserCell_ColorEditor : DialogItemHookHandler
{
	// Dialog Hook replacement function
	static void HookResolve (DialogItemMessage *dimP)
	{
		if (DITEM_MESSAGE_HOOKRESOLVE == dimP->messageType)
		{
			dimP->u.hookResolve.hookHandlerP = new ColorChooserCell_ColorEditor (dimP->db, dimP->dialogItemP);
			dimP->msgUnderstood = true;
		}
	}
	///	<summary> Item Hook Handler Constructor.</summary>
	ColorChooserCell_ColorEditor (MSDialogP dbP, DialogItemP diP) :
		DialogItemHookHandler (dbP, diP) {}
	///	<summary> Item Hook Handler Method overrides
	///	<summary>Get the color index from the DataSheet ListModel and apply it to this ColorChooser.</summary>
	virtual bool 	_OnCreate			(DialogItemCreateArgsR 			init) 		override;
    virtual bool 	_OnStateChanged		(DialogItemStateChangedArgsR 	stateChanged)	override;
	// Other hook handler method overrides...
};

#endif // !defined(HOOK_CLASSES_H_INCLUDED_)
