#include <Mstn/MdlApi/mdl.h>
#include <DgnPlatform/DgnPlatform.h>
#include <Bentley/BeFileName.h>
#include <Mstn/MdlApi/dlogitem.h>
#include <Mstn/MdlApi/ditemlib.fdf>
#include <Mstn/MdlApi/msdialog.fdf>
#include <Mstn/MdlApi/leveltable.fdf>
#include <Mstn/MdlApi/msmisc.fdf>
#include <Mstn/MdlApi/miscilib.fdf>
#include <Mstn/MdlApi/msritem.fdf>
#include <Mstn/MdlApi/mssystem.fdf>
#include <Mstn/MdlApi/mswindow.fdf>
#include <Mstn/MdlApi/workmode.fdf>
#include "datasheet.h"
#include "Hooks.h"
#include "MessageTracer.h"

extern GlobalData  gData;
extern BSIColorDescr*	m_pBackgroundColor;
extern BSIColorDescr*	m_pForegroundColor;

///////////////////////////////////////////////////////////////////////
///	<summary>When user has finished with a ListModel cell editor in a container, invoke this common action.</summary>
void	CloseEditor (MSDialogP	db)
{
	mdlDialog_lastActionTypeSet (db, ACTIONBUTTON_OK);
	mdlWindow_hide ((MSWindowP)db, HideReasonCode::HideWindow, true);
	mdlDialog_closeCommandQueue (db);
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	hookCombo_buildLevelList							|
|																		|
|	description:	Create a ListModel of level names					|
|																		|
|	returns:		ListModel pointer on success						|
|																		|
+--------------------------------------------------------------------- */
ListModelP			hookCombo_buildLevelList
(
void
)
{
	CMessageTracer	msg (L"hookCombo_buildLevelList");
	msg.Output ();

	const int		nColumns	= 1;
	ListModelP		pListModel	= mdlListModel_create (nColumns);
	//	mdlLevelList_getLevelNames creates a ListModel that stores
	//	the level ID in each ListCell
	DgnPlatform::LevelId	levelId;
	ActiveParams::GetValue (levelId, ACTIVEPARAM_LEVEL);
	if (SUCCESS == mdlLevelList_getLevelNamesListModel (pListModel, ACTIVEMODEL, levelId, LEVELLIST_BOLD_USED ))
	{
		for (int row = 0; row != mdlListModel_getRowCount  (pListModel); ++row)
		{
			ListCellP		pCell  = mdlListModel_getCellAtIndexes (pListModel, row, 0);
			WCharCP			display;
			mdlListCell_getDisplayText (pCell, &display);
			MSValueDescr	value;
			mdlListCell_getValue (pCell, value);
			msg = "ListRow "; msg [row];
			msg += " Display"; msg.Quote (display);
			FormatType	fmt	= value.GetFormatType ();
			msg += " Internal formattype="; msg += fmt; msg.Quote ( FMT_UINT == fmt? "Unsigned Int": "other");
			msg.Output ();
		}
	}
	else
	{
		mdlListModel_destroy (pListModel, true);
	}

	return pListModel;
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	listModel_getEditCell								|
|																		|
|	description:	A ListCell is contained by a ListModel that is 		|
|					contained by a ListBox or ComboBox.  The ListModel 	|
|					contains an array of ListCell objects.	A reference |
|					to the currently-selected ListCell is often 		|
|					required to get or set the ListCell's contents.		|
|																		|
|	note:			Don't call from inside a ListBox hook function.		|
|					Instead, get the current row and column from the	|
|					ListBox: 											|
|					mdlDialog_listBoxLastCellClicked ()					|
|					Next, get the associated ListModel:					|
|					mdlDialog_listBoxGetListModelP ()					|
|					Next, get the ListCell at row, colum:				|
|					mdlListModel_getCellAtIndexes ()					|
|																		|
|	usage:			ListCellP  pCell = nullptr;							|
|					pCell	= listModel_getEditCell	(db, rItem);		|
|																		|
|	returns:		Non-const reference to a ListCell on success		|
|																		|
+--------------------------------------------------------------------- */
ListCellP			listModel_getEditCell
(
MSDialogP		 	db,			//	=>	Dialog that contains ComboBox or ListBox
RawItemHdr*			rItem		//	=>	Reference to  ComboBox or ListBox
)
{
	ListCellP		pCell		= nullptr;	//	This is the return value
	DialogItemP		pContainer	= rItem->diP;

	switch (pContainer->type)
	{
		case RTYPE_ListBox:
		{
			ListModelP	pListModel	= mdlDialog_listBoxGetListModelP (rItem);
			int			row = 0;
			int			col = 0;
			int			status		= mdlDialog_listBoxGetEditCell (&row, &col, rItem);
			if (SUCCESS == status)
			{
				pCell	= mdlListModel_getCellAtIndexes (pListModel, row, col);
				CMessageTracer msg (L"mdlListModel_getCellAtIndexes (row="); msg += row; msg += ","; msg += col; msg += ")";
				msg += L" pCell is "; msg += nullptr == pCell? L"nullptr" : L"valid";
				msg.Output ();
			}
			else
			{
				CMessageTracer moan (L"mdlDialog_listBoxGetEditCell (RTYPE_ListBox) failed");
				moan.Advisory ();
			}
			break;
		}
		case RTYPE_ComboBox:
		{
			long		row = 0;
			ListModelP	pComboList	= mdlDialog_comboBoxGetListModelP (rItem);
			int			status		= mdlDialog_comboBoxGetIndex (&row, rItem);
			if (SUCCESS == status)
			{
				ListRowP 	pRow	= mdlListModel_getRowAtIndex (pComboList, row);
				pCell	= mdlListRow_getCellAtIndex (pRow, 0);
				CMessageTracer msg (L"mdlDialog_comboBoxGetIndex (row="); msg += row;
				msg += L" pCell is "; msg += nullptr == pCell? L"nullptr" : L"valid";
				msg.Output ();
			}
			else
			{
				CMessageTracer moan (L"mdlDialog_comboBoxGetIndex (RTYPE_ComboBox) failed");
				moan.Advisory ();
			}
			break;
		}
	}
	return pCell;
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	listCell_setIconByInternalValue						|
|																		|
|	description:	Assuming the long data member of a ListCell holds	|
|					a binary value, set its icon to check/blank or 		|
|					some other visual representation of binary state	|
|																		|
|	returns:		TRUE on success										|
|																		|
+--------------------------------------------------------------------- */
bool				listCell_setIconByInternalValue
(
ListCellP			pCell				//	=>	Modify this cell icon
)
{
	long			flag	= -1;
	bool			rc		= false;
	if (SUCCESS == mdlListCell_getLongValue (pCell, &flag))
	{
		mdlListCell_setIcon (pCell, flag ? ICONID_ColorCheckMark : 0, RTYPE_Icon, mdlSystem_getCurrMdlDesc ());
		//mdlListCell_setIcon (pCell, flag ? ICONID_SmallCheck : ICONID_SmallCross, RTYPE_Icon, mdlSystem_getCurrMdlDesc ());
		rc = true;
	}
	return rc;
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	listModel_getListBoxEditCell						|
|																		|
|	description:	A ListCell is contained by a ListModel that is 		|
|					contained by a ListBox.  The ListModel contains		|
|					an array of ListCell objects.	A reference to the	|
|					currently-selected ListCell is often required to	|
|					obtain or update the ListCell's contents.			|
|																		|
|	note:			If the ListBox is transient 						|
|					you can't use this function							|
|																		|
|	usage:			ListCellP pCell = nullptr;							|
|					pCell	= listModel_getListBoxEditCell	(db, LBOX);	|
|																		|
|	returns:		Non-const reference to a ListCell on success		|
|																		|
+--------------------------------------------------------------------- */
ListCellP			listModel_getListBoxEditCell
(
MSDialogP		  	db,			//	=>	Dialog box that contains our ListBox
ULong 				nIDListBox	//	=>	Resource ID of the ListBox
)
{
	DialogItemP		pLBox		= mdlDialog_itemGetByTypeAndId (db, RTYPE_ListBox, nIDListBox, 0);

	return (nullptr == pLBox) ? nullptr : listModel_getEditCell (db, pLBox->rawItemP);
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	listModel_getComboBoxEditCell						|
|																		|
|	description:	A ListCell is contained by a ListModel that is 		|
|					contained by a ComboBox.  The ListModel contains	|
|					an array of ListCell objects.	A reference to the	|
|					currently-selected ListCell is often required to	|
|					obtain or update the ListCell's contents.			|
|																		|
|	note:			If the ComboBox is transient (created for editing	|
|					this ListCell), you can't use this function, but	|
|					listModel_getEditCell() works from the RawItemHdr*	|
|																		|
|	usage:			ListCell *pCell = NULL;								|
|					pCell	= listModel_getComboBoxEditCell	(db, CBOX);	|
|																		|
|	returns:		Non-const reference to a ListCell on success		|
|																		|
+--------------------------------------------------------------------- */
ListCellP			listModel_getComboBoxEditCell
(
MSDialogP   		db,			//	=>	Dialog box that contains our ComboBox
ULong 				nIDComboBox	//	=>	Resource ID of the ComboBox
)
{
	DialogItem*		pComboBox	= mdlDialog_itemGetByTypeAndId (db, RTYPE_ComboBox, nIDComboBox, 0);
	return (nullptr == pComboBox) ? nullptr : listModel_getEditCell (db, pComboBox->rawItemP);
}
/* ---------------------------------------------------------------------+
|																		|
|	function name:	hookClrPick_cellEditor								|
|																		|
|	events handled:														|
|																		|
|	description:	Provides a Color picker editor for a ListCell		|
|																		|
+--------------------------------------------------------------------- */
void	hookClrPick_cellEditor
(
DialogItemMessage	*dimP
)
{
    dimP->msgUnderstood = TRUE;

    switch (dimP->messageType)
    {
      	case DITEM_MESSAGE_CREATE:
		{
            ListCellP	pCell		= nullptr;
			bool		valChanged	= false;

			if (pCell	= listModel_getListBoxEditCell (dimP->db, LISTBOXID_CellsSheet))
			{
				long	lVal = 0;
				mdlListCell_getLongValue (pCell, &lVal);
				MSValueDescr	value (lVal);
				mdlDialog_itemSetValue (&valChanged, value, dimP->db, dimP->itemIndex);
				mdlDialog_itemSetState (&valChanged, dimP->db, dimP->itemIndex);
			}
 	    	dimP->u.create.createFailed = FALSE;
	    	break;
	    }
      	case DITEM_MESSAGE_STATECHANGED:
		{
            ListCellP		pCell		= nullptr;
			int				Color		= -1;
			MSValueDescr	value;
			mdlDialog_itemGetValue (value, dimP->db, dimP->itemIndex);
			Color = (int)value.GetLong ();
			if ((pCell	= listModel_getListBoxEditCell (dimP->db, LISTBOXID_CellsSheet))
				&&
				SUCCESS == mdlListCell_setLongValue (pCell, value.GetLong ()))
			{
				m_pForegroundColor = mdlColorPal_getColorDescr (nullptr, (int)value.GetLong ());
				mdlListCell_setIconColorOverrides (pCell, m_pBackgroundColor, m_pForegroundColor);
			}
            break;
	    }
      	case DITEM_MESSAGE_DESTROY:
		{
	    	break;
	    }
      	default:
	    	dimP->msgUnderstood = FALSE;
	    	break;
	}
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	hookText_cellColorEditor							|
|																		|
|	description:	A Text item lets you type a numeric Color index		|
|					to set a cell long value.  The long value is 		|
|					rendered as the Color of the cell's icon			|
|																		|
|	returns:		Nothing												|
|																		|
+--------------------------------------------------------------------- */
void 				hookText_cellColorEditor
(
DialogItemMessage*	dimP
)
{
    dimP->msgUnderstood = true;

    switch (dimP->messageType)
	{
		case DITEM_MESSAGE_CREATE:
		{
			long			initialise = -1;

			if (ListCellP	pCell	= listModel_getListBoxEditCell (dimP->db, LISTBOXID_CellsSheet))
			{
				mdlListCell_getLongValue (pCell, &initialise);
				WChar			value [32];
				swprintf (value, L"%d", (int)initialise);
				bool			changed	= false;
				mdlDialog_itemSetStringValue (&changed, value, dimP->db, dimP->itemIndex);
				mdlDialog_itemSetState (&changed, dimP->db, dimP->itemIndex);
			}
			break;
		}
		case DITEM_MESSAGE_QUEUECOMMAND:
		{
			int				color = -1;
			MSValueDescr	value;
			mdlDialog_itemGetValue (value, dimP->db, dimP->itemIndex);
			color = _wtoi (value.GetWCharCP ());
			ListCellP		pCell	= nullptr;
			if	((pCell	= listModel_getListBoxEditCell (dimP->db, LISTBOXID_CellsSheet))
				&&
				(SUCCESS == mdlListCell_setLongValue (pCell, _wtol (value.GetWCharCP ()))))
			{
				m_pForegroundColor = mdlColorPal_getColorDescr (NULL, color);
				mdlListCell_setIconColorOverrides (pCell, m_pBackgroundColor, m_pForegroundColor);
			}
			break;
		}
		default:
			dimP->msgUnderstood = FALSE;
			break;
	}
}
/*----------------------------------------------------------------------+*//**
* A text item that is used as an editor field.								    	    *
* @bsimethod myapp_dateItemHook   					*					    	*
* @param 	dimP     the dialog item message sent to the item	    	*
* @return 	!!!what the return value means			    	*
*								        *
* Author:   MarkAnderson 				04/01 		*
*								        *
+----------------------------------------------------------------------*/
void datasheet_textItemHook
(
DialogItemMessage*	dimP   /* => a ptr to a dialog item message */
)
{
    dimP->msgUnderstood = true;

    switch (dimP->messageType)
	{
		case DITEM_MESSAGE_INIT:
      	{
            //get the value in the listbox then transfer it to the text box.
            //this should be unnecessary for the simple case since a text item is "created"
            //behind the scenes.
            DialogItemP		diP 	= mdlDialog_itemGetByTypeAndId (dimP->db, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
            ListModelP		pModel	= mdlDialog_listBoxGetListModelP (diP->rawItemP);
            int             rowI, colI;
            mdlDialog_listBoxGetEditCell (&rowI, &colI, diP->rawItemP );
            ListCellP		pCell	= mdlListModel_getCellAtIndexes (pModel, rowI, colI);
            MSValueDescr	value;
            mdlListCell_getValue (pCell, value);
            //set the string in the text item.
            //mdlDialog_itemSetStringValue (&valChanged,&valdescr.value.charPFormat,dimP->db,dimP->itemIndex);
            //this one worked.
	        mdlDialog_rItemValueSet (nullptr, value, dimP->dialogItemP->rawItemP);
            //this doesnot work!
            //mdlDialog_itemSetValue (nullptr,valdescr.formatType,&valdescr.value,nullptr,dimP->db,dimP->itemIndex);
			break;
		}
		case DITEM_MESSAGE_STATECHANGED:
		if  (dimP->u.stateChanged.reallyChanged )
		{
			int             rowI, colI;
			MSValueDescr	value;
			mdlDialog_itemGetValue (value, dimP->db, dimP->itemIndex);

			if  (0 != wcsicmp (value.GetWCharCP (), L"x"))
			{
				DialogItemP	diP		= mdlDialog_itemGetByTypeAndId (dimP->db, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
				ListModelP	pModel	= mdlDialog_listBoxGetListModelP (diP->rawItemP);
				mdlDialog_listBoxGetEditCell (&rowI, &colI, diP->rawItemP);
				ListCellP	pCell = mdlListModel_getCellAtIndexes (pModel, rowI, colI);
				mdlListCell_setStringValue (pCell, value.GetWCharCP (), true);
			}
			break;
		}
		default:
			dimP->msgUnderstood = false;
			break;
	}
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	hookTBtn_cellBoolEditor								|
|																		|
|	description:	Manage the transient toggle button for LBox col 3	|
|																		|
|	author:			LA Solutions Ltd									|
|																		|
|	returns:		Nothing												|
|																		|
+--------------------------------------------------------------------- */
 void 				hookTBtn_cellBoolEditor
(
DialogItemMessage*	dimP
)
{
    dimP->msgUnderstood = true;
    switch  (dimP->messageType)
	{
	case DITEM_MESSAGE_CREATE:
	{
		bool			changed	= false;
		ListCellP		pCell	= listModel_getListBoxEditCell (dimP->db, LISTBOXID_CellsSheet);
		if (pCell)
		{
			long			val;
			mdlListCell_getLongValue (pCell, &val);
			MSValueDescr	value (val);
     		mdlDialog_itemSetValue (&changed, value, dimP->db, dimP->itemIndex);
     	}
		break;
	}
	case DITEM_MESSAGE_BUTTON:
	{
		if ((BUTTONTRANS_UP == dimP->u.button.buttonTrans)
			&&
			(DATAPNT == dimP->u.button.buttonNumber)
			&&
			(dimP->u.button.clicked))
		{
			//msg ="ToggleButton BUTTON clicked ", msg += (1 == dimP->u.button.upNumber)? "once": "twice";
			//msg.Output ();
		}
		break;
	}
	default:
		dimP->msgUnderstood = false;
		break;
	}
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	hookCombo_cellBoolEditor							|
|																		|
|	description:	A BoolInt cell editor displays a ComboBox to user	|
|					that shows Yes|No options and sets cell long value	|
|					to 0 or 1											|
|																		|
|	returns:		TRUE on success										|
|																		|
+--------------------------------------------------------------------- */
void    			hookCombo_cellBoolEditor
(
DialogItemMessage*	dimP
)
{
	RawItemHdr*		pThis	= dimP->dialogItemP->rawItemP;

    dimP->msgUnderstood = true;
    switch  (dimP->messageType)
	{
		case DITEM_MESSAGE_CREATE:
	    {
			//	This is the ComboBox internal ListModel
			//	Add one row for Yes
	    	ListModelP	pComboList	= mdlListModel_create (1);
			ListRowP	pRow 		= mdlListRow_create (pComboList);
			ListCellP	pCell 		= mdlListRow_getCellAtIndex (pRow, 0);
			mdlListCell_setStringValue (pCell, L"Yes", true);
			mdlListModel_addRow (pComboList, pRow);
			//	Add one row for No
			pRow	= mdlListRow_create (pComboList);
			pCell	= mdlListRow_getCellAtIndex (pRow, 0);
			mdlListCell_setStringValue (pCell, L"No", true);
			mdlListModel_addRow (pComboList, pRow);
	    	mdlDialog_comboBoxSetListModelP (pThis, pComboList);
		    break;
	    }
		case DITEM_MESSAGE_STATECHANGED:
	    {
            long     	yesNo			= -1;
            mdlDialog_comboBoxGetIndex (&yesNo, pThis);
			//	Get the corresponding cell in the DataSheet ListBox
            ListCellP	pCell = listModel_getListBoxEditCell (dimP->db, LISTBOXID_CellsSheet);
            if (pCell)
            {
            	long		value		= (long)(0 == yesNo ? 1: 0);
            	mdlListCell_setLongValue (pCell, value);
            	listCell_setIconByInternalValue (pCell);
            }
            break;
		}
		case DITEM_MESSAGE_DESTROY:
	    {
			ListModelP	pListModel = mdlDialog_comboBoxGetListModelP (pThis);
			if (pListModel)
				mdlListModel_destroy (pListModel, true);
			break;
	    }

		default:
			dimP->msgUnderstood = false;
			break;
	}
}
/* ---------------------------------------------------------------------+
|																		|
|	function name:	hookCombo_cellLevelEditor							|
|																		|
|	events handled:	CREATE        -- populate with level list			|
|					STATECHANGED  -- set value of ListCell 				|
|																		|
|	description:	Handles level list for a ListCell					|
|																		|
|																		|
+--------------------------------------------------------------------- */
void				hookCombo_cellLevelEditor
(
DialogItemMessage*	dimP
)
{
	RawItemHdr*		pThis	= dimP->dialogItemP->rawItemP;

    dimP->msgUnderstood = true;

    switch (dimP->messageType)
    {
      	case DITEM_MESSAGE_CREATE:
		{
			int				i			= -1;
			//	This is this ComboBox's internal ListModel
			ListModelP		pComboList = hookCombo_buildLevelList ();
	    	mdlDialog_comboBoxSetListModelP (pThis, pComboList);
	    	//	Synchronise combo display with current ListCell contents from DataSheet
            DialogItemP		pListBox	= mdlDialog_itemGetByTypeAndId (dimP->db, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
            ListModelP 		pDataList	= mdlDialog_listBoxGetListModelP (pListBox->rawItemP);
            int				rowI = -1, colI = -1;
            //	Get the DataSheet ListBox current cell
            mdlDialog_listBoxGetEditCell (&rowI, &colI, pListBox->rawItemP );
            ListCellP		pCell = mdlListModel_getCellAtIndexes (pDataList, rowI, colI);
	    	WString			level;
	    	mdlListCell_getDisplayTextWString (pCell, level);
	    	if (!level.empty ())
	    	{	//	Attempt to match a row in this ComboBox
				for (i = 0; i != mdlListModel_getRowCount (pComboList); ++i)
				{
					pCell	= mdlListModel_getCellAtIndexes (pComboList, i, 0);
					MSValueDescr	value;
	    			mdlListCell_getValue (pCell, value);
          			WChar			wcLevel		[Bentley::DgnPlatform::MAX_LEVEL_NAME_LENGTH];
					mdlLevel_getName  (wcLevel, Bentley::DgnPlatform::MAX_LEVEL_NAME_LENGTH, ACTIVEMODEL, value.GetLong ());
					if (0 == wcsicmp (level.c_str (), wcLevel))
					{
						mdlDialog_comboBoxSetIndex (i, pThis);
						break;
					}
				}
			}
	    	break;
	    }
		case DITEM_MESSAGE_DESTROY:
	    {
			ListModelP	pListModel = mdlDialog_comboBoxGetListModelP (pThis);
			if (pListModel)
				mdlListModel_destroy (pListModel, true);
			break;
	    }
		case DITEM_MESSAGE_STATECHANGED:
	    {
			//	Get user selection from ComboBox ListModel
			long		row	= -1;
            mdlDialog_comboBoxGetIndex (&row, pThis);
            ListModelP	pComboList	= mdlDialog_comboBoxGetListModelP (pThis);
            ListRowP	pRow		= mdlListModel_getRowAtIndex (pComboList, row);
            ListCellP	pCell 		= mdlListRow_getCellAtIndex (pRow, 0);
            WString 	levelName;
            mdlListCell_getDisplayTextWString (pCell, levelName);
			//	Apply value to DataSheet ListModel
            DialogItemP	pListBox	= mdlDialog_itemGetByTypeAndId (dimP->db, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
            ListModelP 	pDataList	= mdlDialog_listBoxGetListModelP (pListBox->rawItemP);
            int			rowI = -1, colI = -1;
            mdlDialog_listBoxGetEditCell (&rowI, &colI, pListBox->rawItemP );
            pCell = mdlListModel_getCellAtIndexes (pDataList, rowI, colI);
			if (pCell)
			{
				 mdlListCell_setDisplayText (pCell, levelName.c_str ());
		 		//mdlListCell_setStringValue (pCell, levelName.c_str (), true);
			}
            break;
		}
      	default:
	    	dimP->msgUnderstood = false;
	    	break;
	}
}
/*----------------------------------------------------------------------+*//**
* the combobox that will be an editor for a column of thelistbox								    	    *
* @bsimethod hookCombo_cellTextEditor   					*					    	*
* @param 	dimP     The dialog message sent to the item	    	*
* @return 	!!!what the return value means			    	*
*								        *
* Author:   MarkAnderson 				04/01 		*
*								        *
+-----------------------------------------------------------------------*/
void    			hookCombo_cellTextEditor
(
DialogItemMessage*	dimP
)
{
	RawItemHdr*		pThis	= dimP->dialogItemP->rawItemP;

    dimP->msgUnderstood = true;
    switch  (dimP->messageType)
	{
		case DITEM_MESSAGE_CREATE:
	    {
			//	This is the ComboBox internal ListModel
	    	ListModelP	pListModel = mdlListModel_create (1);
	    	for (int x = 0; x <= 10 ; ++x)
	    	{
				WChar    	xs[4];
				ListRowP	pRow	= mdlListRow_create (pListModel);
				ListCellP	pCell	= mdlListRow_getCellAtIndex (pRow, 0);
				swprintf (xs, L"x%ld", x);
				mdlListCell_setStringValue (pCell, xs, true);
				mdlListModel_addRow (pListModel, pRow);
	    	}
	    	mdlDialog_comboBoxSetListModelP (pThis, pListModel);
	    }
	    break;

		case DITEM_MESSAGE_STATECHANGED:
	    {
			WString		string;
			//	Get the value of the current ListCell from this ComboBox
            ListCellP	pComboCell = listModel_getEditCell (dimP->db, pThis);
            mdlListCell_getStringValueWString  (string, pComboCell);
			//	Assign the value to the current ListCell in the ListBox
            ListCellP	pDataCell = listModel_getListBoxEditCell (dimP->db, LISTBOXID_CellsSheet);
            mdlListCell_setStringValue (pDataCell, string.c_str (), true);
            break;
		}

		case DITEM_MESSAGE_DESTROY:
	    {
			ListModelP	pListModel = mdlDialog_comboBoxGetListModelP (pThis);
			if (pListModel)
				mdlListModel_destroy (pListModel, true);
	    }
	    break;

		default:
			dimP->msgUnderstood = false;
			break;
    	}
    }
/*----------------------------------------------------------------------+*//**
* creates the listmodel that the listbox will use.								    	    *
* @bsimethod createListModel   						*					    	*
* @param 	nCols     number of columns in the listbox	    	*
* @return 	the listmodel to connect to the listbox			    	*
*								        *
* Author:   MarkAnderson 				04/01 		*
*								        *
+----------------------------------------------------------------------*/
ListModelP			createListModel
(
int					nCols
)
{
	ListModelP		pListModel = mdlListModel_create (nCols);
	const int		nRows		= 6;
	const bool		EditDisplay			{ false };
	const bool		UpdateDisplay		{ true };
	const bool		SetDisplayString	{ true };
	for (int rowIndex = 0; rowIndex != nRows; ++rowIndex)
	{
		ListRowP	pRow = mdlListRow_create (pListModel);
		for (int colIndex = 0; colIndex < nCols; ++colIndex)
		{
			WChar		value 	[32];
			swprintf (value, L"value %d.%d", rowIndex, colIndex);
			ListCellP	pCell = mdlListRow_getCellAtIndex (pRow, colIndex);

			switch (colIndex)
			{
			case colSimpleTextEditor:
				//	Column 1 shows plain text with text editor
				mdlListCell_setStringValue (pCell, value, SetDisplayString);
				mdlListCell_setEditor (pCell, RTYPE_Text, TEXTID_UserInfo,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				//mdlListCell_setEditor (pCell, RTYPE_Container, CONTAINERID_TextEditor, mdlSystem_getCurrMdlDesc (), FALSE, TRUE);
				break;
			case colComboTextEditor:
				//	Column 2 shows text provided by a ComboBox editor
				mdlListCell_setStringValue (pCell, value, SetDisplayString);
				mdlListCell_setEditor (pCell, RTYPE_ComboBox, COMBOBOXID_DataSheetEditor,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			case colLevelCombo:
			{
				//	Column 5 displays level edited using a ComboBox
				mdlListCell_setStringValue (pCell, value, SetDisplayString);
				//	COMBOBOXID_Levels uses HOOKITEMID_ComboBoxLevels
				mdlListCell_setEditor (pCell, RTYPE_ComboBox, COMBOBOXID_Levels,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			}
			case colComboCheckBox:
			{
				//	Column 3 displays a cross or tick icon and is edited
				//	either using a ComboBox to display Yes or No
				//	or by a double-click handled by the ListBox hook function
				const long	flag = (long)(rowIndex % 2 ? 1 : 0);
				mdlListCell_setLongValue (pCell, flag);
				listCell_setIconByInternalValue (pCell);

				mdlListCell_setEditor (pCell, RTYPE_ComboBox, COMBOBOXID_DataSheetYesNo,
						mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			}
			case colColourSwatch:
			{
				//	Column 4 displays a Color patch and is edited
				//	using a ColorChooser in a Container
				//	or a text item
				const long 	color = (rowIndex * nCols + colIndex) % 256;
				mdlListCell_setLongValue (pCell, color);
				mdlListCell_setIcon (pCell, ICONID_ColorPatch, RTYPE_Icon, mdlSystem_getCurrMdlDesc ());
				m_pForegroundColor = mdlColorPal_getColorDescr (nullptr, color);
				mdlListCell_setIconColorOverrides (pCell, m_pBackgroundColor, m_pForegroundColor);

				//	Text widget works OK for editing numeric Color palette index
				//mdlListCell_setEditor (pCell, RTYPE_Text, TEXTID_Color,
				//		mdlSystem_getCurrMdlDesc (), FALSE, TRUE);
				//mdlListCell_setEditor (pCell, RTYPE_ColorPicker, COLORPICKERID_DataSheetClr,
				//		mdlSystem_getCurrMdlDesc (), FALSE, TRUE);
				//  CONTAINERID_ColorChooser contains COLORCHOOSERID_Color
				mdlListCell_setEditor (pCell, RTYPE_Container, CONTAINERID_ColorChooser, mdlSystem_getCurrMdlDesc (), EditDisplay, UpdateDisplay);
				break;
			}
			}
		}
		mdlListModel_addRow (pListModel, pRow);
	}

	return pListModel;
}
/*----------------------------------------------------------------------+*//**
* hook function for the listbox								    	    *
* @bsimethod listBoxItemHook   						*					    	*
* @param 	*dimP     Dialog Item Message	    	*
* @return 	!!!what the return value means			    	*
*								        *
* Author:   MarkAnderson 				04/01 		*
*								        *
+----------------------------------------------------------------------*/
void 				listBoxItemHook
(
DialogItemMessage*	dimP
)
{
	int	         	row = -1, col = -1;
    int          	status;

    dimP->msgUnderstood = true;

    switch  (dimP->messageType)
	{
	    case DITEM_MESSAGE_CREATE:
    	{
	        ListModelP	pListModel = createListModel (nColumns);
	        mdlDialog_listBoxSetListModelP (dimP->dialogItemP->rawItemP, pListModel, nColumns);
	    	break;
   		}
	    case DITEM_MESSAGE_DESTROY:
   		{
	        ListModelP	pListModel = mdlDialog_listBoxGetListModelP (dimP->dialogItemP->rawItemP);
	        if (pListModel)
	        	mdlListModel_destroy (pListModel, true);
	    	break;
  		}
	    case DITEM_MESSAGE_BUTTON:
     	{
	        if  (BUTTONTRANS_UP == dimP->u.button.buttonTrans)
		 	{
		        if (SUCCESS==(status = mdlDialog_listBoxLastCellClicked (&row, &col, dimP->dialogItemP->rawItemP)))
           		{
                    mdlDialog_listBoxEditCell (dimP->dialogItemP->rawItemP, row, col);
				}
			}
	    	break;
     	}
		case DITEM_MESSAGE_STATECHANGED:
      	{
            if  (dimP->u.stateChanged.reallyChanged)
			{
			}
            break;
		}
		default:
			dimP->msgUnderstood = false;
		break;
	}
}
/* ---------------------------------------------------------------------+
|																		|
|	function name:	hookColorChooser_cellEditor							|
|																		|
|	events handled:														|
|																		|
|	description:														|
|																		|
|																		|
+--------------------------------------------------------------------- */
void				hookColorChooser_cellEditor
(
DialogItemMessage*	dimP
)
{
    dimP->msgUnderstood = true;

    switch (dimP->messageType)
	{
	case DITEM_MESSAGE_CREATE:
	{
		///	<summary>Get the color index from the DataSheet ListModel and apply it to this ColorChooser.</summary>
	    int		    	row = -1, col = -1;
	    //	The dialog that owns this widget is a Container,
	    //	which is not the same as the Dialog that contains the ListBox
	  	MSDialogP		dbP		= mdlDialog_find (DIALOGID_SpreadSheet, 0);
		DialogItemP		pLBox	= mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	    mdlDialog_listBoxGetEditCell (&row, &col, pLBox->rawItemP);
	    //	The ColorChooser is in a Container which is owned by
	    //	the real dialog box
	    ListCellP		pCell = listModel_getListBoxEditCell (dbP, LISTBOXID_CellsSheet);
	    if (nullptr == pCell)
	    {
			CMessageTracer moan (L"ColorChooser_CREATE  pCell == NULL dialogID="); moan += dimP->dialogId;
			moan.Advisory ();
	    }
	    else
	    {
			long			lVal;
		    mdlListCell_getLongValue (pCell, &lVal);
			bool			changed;
			MSValueDescr	value (lVal);
	    	mdlDialog_itemSetValue (&changed, value, dimP->db, dimP->itemIndex);
		}
	    break;
	}
	case DITEM_MESSAGE_STATECHANGED:
	{
		///	<summary>Copy the colour index from the ColorChooser to the DataSheet ListModel and set its colour swatch.</summary>
	    ///	<remarks>	The dialog that owns this ColorChooser is a Container.
	    ///	The pointer dimP->db references the Container,
	    ///	which is not the same as the Dialog that contains the ListBox.</remarks>
	  	MSDialogP		dbP		= mdlDialog_find (DIALOGID_SpreadSheet, NULL);
		DialogItem*		pLBox	= mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	    int		    	row 	= -1, col = -1;
	    ListModelP		pModel	= mdlDialog_listBoxGetListModelP (pLBox->rawItemP);
		//	Get a reference to the ListBox current ListCell
	    mdlDialog_listBoxGetEditCell (&row, &col, pLBox->rawItemP);
	    ListRowP	pRow  = mdlListModel_getRowAtIndex (pModel, row);
	    ListCellP	pCell = mdlListRow_getCellAtIndex (pRow, col);
		//	Get this ColorChooser's Color index
		MSValueDescr	value;
		mdlDialog_itemGetValue (value, dimP->db, dimP->itemIndex);
		if (0 <= value.GetLong () && value.GetLong () < 256)
		{
			mdlListCell_setLongValue (pCell, value.GetLong ());
			m_pForegroundColor = mdlColorPal_getColorDescr (NULL, (int)value.GetLong ());
			//	Set the new icon Color and store its Color index in the ListCell
			mdlListCell_setIconColorOverrides (pCell, m_pBackgroundColor, m_pForegroundColor);
			mdlDialog_itemDraw (dbP, pLBox->itemIndex);
		}
		//	Inform the ListCell that we've finished editing
		CloseEditor (dimP->db);
	    break;
	}

	default:
	    dimP->msgUnderstood = false;
	    break;
	}
}
/*----------------------------------------------------------------------+
|									|
| name		textstyle_colorByLevelHook				|
|									|
+----------------------------------------------------------------------*/
void	textstyle_colorByLevelHook
(
DialogItemMessage   *dimP
)
{
    dimP->msgUnderstood = true;

    switch (dimP->messageType)
	{
	case DITEM_MESSAGE_CREATE:
	{
	    RscId	 		iconId 		{ -76 };
        UInt32 			iconType 	{ RTYPE_Icon };
	    mdlDialog_pushButtonSetIcon (dimP->dialogItemP->rawItemP, nullptr, &iconId, &iconType, nullptr);
        UInt32           activeLevel;
        ActiveParams::GetValue (activeLevel, ACTIVEPARAM_LEVEL);
	    UInt	     	color;
	    mdlLevel_getElementColor (&color, ACTIVEMODEL, activeLevel);
	    BSIColorDescr*	pFgColorDesc	= nullptr;
	    mdlColorPal_getElemColorDescrByModelRef (&pFgColorDesc, color, ACTIVEMODEL, 0);
	    mdlDialog_pushButtonSetIconColors (dimP->dialogItemP->rawItemP, &pFgColorDesc, nullptr);

	    if	(mdlCapability_isEnabled (CAPABILITY_BYLEVEL))
		    mdlDialog_rItemEnabledStateSet (dimP->dialogItemP->rawItemP, true, false);

	    break;
	}
    case DITEM_MESSAGE_BUTTON:
	{

	    if (! (dimP->u.button.buttonTrans == BUTTONTRANS_UP))
			break;

        MSDialogP		dbP 	= mdlDialog_find (DIALOGID_SpreadSheet,nullptr);
        DialogItemP		listboxP = mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
        ListModelP		pModel = mdlDialog_listBoxGetListModelP (listboxP->rawItemP);

        int        	 	row,col;
        mdlDialog_listBoxGetEditCell (&row,&col,listboxP->rawItemP );
        ListCellP		pCell = mdlListModel_getCellAtIndexes (pModel,row,col);
		WString			string (L"By Level");
        mdlListCell_setStringValue (pCell, string.c_str (), true);
		CloseEditor (dimP->db);

	    break;
   }

	default:
	    dimP->msgUnderstood = false;
	    break;
	}
}
/*----------------------------------------------------------------------+*//**
* the combobox that will be an editor for a column of thelistbox								    	    *
* @bsimethod comboboxItemHook   					*					    	*
* @param 	dimP     The dialog message sent to the item	    	*
* @return 	!!!what the return value means			    	*
*								        *
* Author:   MarkAnderson 				04/01 		*
*								        *
+-----------------------------------------------------------------------*/
void    				comboboxItemHook
(
DialogItemMessage*		dimP
)
{
    dimP->msgUnderstood = true;
    switch  (dimP->messageType)
	{
	    case DITEM_MESSAGE_CREATE:
		{
	        int			x;
            ListModelP	pListModel = mdlListModel_create (1);
	        for (x = 0; x <= 10 ; ++x)
	    	{
		        WChar    	xs[4];
		        ListRowP	pRow	= mdlListRow_create (pListModel);
		        ListCellP 	pCell	= mdlListRow_getCellAtIndex (pRow, 0);
		        swprintf (xs, L"x%ld", x);
		        mdlListCell_setStringValue (pCell, xs, true);
		        mdlListModel_addRow (pListModel, pRow);
	    	}
	        mdlDialog_comboBoxSetListModelP (dimP->dialogItemP->rawItemP,pListModel);
		    break;
		}

	    case DITEM_MESSAGE_STATECHANGED:
	 	{
            long         row;
            int          rowI, colI;
			//	Get user selection from ComboBox ListModel
            mdlDialog_comboBoxGetIndex (&row, dimP->dialogItemP->rawItemP);
            ListModelP	pComboList	= mdlDialog_comboBoxGetListModelP (dimP->dialogItemP->rawItemP);
            ListRowP	pRow		= mdlListModel_getRowAtIndex (pComboList, row);
            ListCellP	pCell 		= mdlListRow_getCellAtIndex (pRow, 0);
            WString 	s;
            mdlListCell_getStringValueWString  (s, pCell);
			//	Apply value to DataSheet ListModel
            DialogItemP	diP 		= mdlDialog_itemGetByTypeAndId (dimP->db, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
            ListModelP 	pDataList	= mdlDialog_listBoxGetListModelP (diP->rawItemP);
            mdlDialog_listBoxGetEditCell (&rowI, &colI, diP->rawItemP );
            pCell = mdlListModel_getCellAtIndexes (pDataList, rowI, colI);
            mdlListCell_setStringValue (pCell, s.c_str (), true);
            //	Call the following only if this widget is hosted in a Container.
			//CloseEditor (dimP->db);
            break;
    	}

	    case DITEM_MESSAGE_DESTROY:
		{
            ListModelP	pListModel	= mdlDialog_comboBoxGetListModelP (dimP->dialogItemP->rawItemP);
	        if (pListModel)
	        	mdlListModel_destroy (pListModel, true);
	   	 	break;
		}

	    default:
	    {
	    	dimP->msgUnderstood = false;
	    	break;
		}
	}
}
