/*----------------------------------------------------------------------+
|																		|
| MessageTracer Copyright 2016, LA Solutions Ltd						|
| All rights reserved.													|
|																		|
| Limited permission is hereby granted to reproduce and modify this		|
| copyrighted material provided that this notice is retained			|
| in its entirety in any such reproduction or modification.				|
|																		|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   MessageTracer - MessageTracer source code. 							|
|                                                                       |
+----------------------------------------------------------------------*/

#include 	"MessageTracer.h"
#include    <Mstn/MdlApi/MdlApi.h>
#include	<DgnPlatform/NotificationManager.h>

USING_NAMESPACE_BENTLEY_DGNPLATFORM

///////////////////////////////////////////////////////////////////////
//	Implementation
///////////////////////////////////////////////////////////////////////
//	static
std::wstring	CMessageTracer::Convert			(char const*		s)
{
	const size_t	len		= strlen (s);
	WChar*			temp	= new WChar [1 + len]; *temp	= 0;
	mbstowcs (temp, s, len);
	 *(len + temp)	= 0;
	std::wstring	ws (temp);
	delete [] temp;
	return ws;
}
///////////////////////////////////////////////////////////////////////
//	static
std::wstring	CMessageTracer::Convert			(std::string const&	s)
{
	return Convert (s.c_str ());
}
///////////////////////////////////////////////////////////////////////
//	Output to MicroStation's message center.  If verbose_ is empty,
//	send terse_ to the message center list and detail window;
//	if verbose_ is not empty, send it to the detail window
void			CMessageTracer::Output	(DgnPlatform::OutputMessagePriority				priority)	const
{
	const NotifyMessageDetails	notify (priority, terse_.c_str (), verbose_.empty ()? terse_.c_str (): verbose_.c_str (), 0, DgnPlatform::OutputMessageAlert::None);
	NotificationManager::OutputMessage (notify);
}
///////////////////////////////////////////////////////////////////////
//	Output to MicroStation's message center.  If verbose_ is empty,
//	send terse_ to the message center list and detail window;
//	if verbose_ is not empty, send it to the detail window
void			CMessageTracer::Warning	(DgnPlatform::OutputMessagePriority				priority,
										bool				pop)	const
{
	const NotifyMessageDetails	notify (priority, terse_.c_str (), verbose_.empty ()? terse_.c_str (): verbose_.c_str (), 0, pop? DgnPlatform::OutputMessageAlert::Dialog: DgnPlatform::OutputMessageAlert::None);
	NotificationManager::OutputMessage (notify);
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator +=	(double					d)
{
	AppendDbl (d);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator +=	(bool					b)
{
	Append (b);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const char*			msg)
{
	terse_		= Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const std::string&		msg)
{
	terse_		= Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const WChar*			msg)
{
	terse_		= msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const std::wstring&	msg)
{
	terse_		= msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(WStringCR				msg)
{
	terse_		= msg.c_str ();
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const char*			msg)
{
	terse_	+=	Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const std::string&		msg)
{
	terse_	+=	Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const WChar*			msg)
{
	terse_	+=	msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const std::wstring&	msg)
{
	terse_	+=	msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const Bentley::WString&	msg)
{
	terse_	+=	msg.c_str ();
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(UInt32						number)
{
	Append (number);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(Int32						number)
{
	Append (number);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator []	(int						i)
{
	Bracket (i);
	return *this;
}

///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Space	(int count)
{
	const std::wstring	Space (L" ");
	for (int i = 0; i != count; ++i)
	{
		terse_	+= Space;
	}
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::NewLine	()
{
	const std::wstring	EOL (L"\n");
	terse_	+= EOL;

}
///////////////////////////////////////////////////////////////////////
std::wstring		CMessageTracer::Wrap (WCharCP msg, WCharCP open, WCharCP close)
{
	std::wstring	s (open);
	s += msg;
	s += close;
	return s;
}
///////////////////////////////////////////////////////////////////////
std::string		CMessageTracer::Wrap (const char* msg, const char* open, const char* close)
{
	std::string	s (open);
	s += msg;
	s += close;
	return s;
}
///////////////////////////////////////////////////////////////////////
std::string		CMessageTracer::Wrap (std::string const& msg, const char* open, const char* close)
{
	return Wrap (msg.c_str (), open, close);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Bracket	(std::wstring const&		msg, BracketOption bracket)
{
	Bracket (msg.c_str (), bracket);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Bracket	(WStringCR		msg, BracketOption bracket)
{
	Bracket (msg.c_str (), bracket);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Bracket	(WCharCP		msg, BracketOption bracket)
{
	switch (bracket)
	{
		case BracketOption::Parenthesis:
			terse_		+=  Wrap (msg, L" (", L")");
			break;
		case BracketOption::Square:
			terse_		+=  Wrap (msg, L" [", L"]");
			break;
		case BracketOption::Brace:
			terse_		+=  Wrap (msg, L" {", L"}");
			break;
		case BracketOption::SingleQuote:
			terse_		+=  Wrap (msg, L" '", L"'");
			break;
		case BracketOption::DoubleQuote:
			terse_		+=  Wrap (msg, L" \"", L"\"");
			break;
	}
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Bracket	(std::string const&		msg, BracketOption bracket)
{
	Bracket (msg.c_str (), bracket);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Bracket	(const char*		msg, BracketOption bracket)
{
	switch (bracket)
	{
		case BracketOption::Parenthesis:
			terse_		+= Convert (Wrap (msg, " (", ")"));
			break;
		case BracketOption::Square:
			terse_		+= Convert (Wrap (msg, " [", "]"));
			break;
		case BracketOption::Brace:
			terse_		+= Convert (Wrap (msg, " {", "}"));
			break;
		case BracketOption::SingleQuote:
			terse_		+= Convert (Wrap (msg, " '", "'"));
			break;
		case BracketOption::DoubleQuote:
			terse_		+= Convert (Wrap (msg, " \"", "\""));
			break;
	}
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const char*		msg)
{
	Bracket (msg, BracketOption::SingleQuote);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const std::string&	msg)
{
	Bracket (msg.c_str (), BracketOption::SingleQuote);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const WChar*			msg)
{
	Bracket (msg, BracketOption::SingleQuote);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const std::wstring&	msg)
{
	Bracket (msg.c_str (), BracketOption::SingleQuote);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(WStringCR				msg)
{
	Bracket (msg.c_str (), BracketOption::SingleQuote);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Bracket	(int					number)
{
	WChar			temp	[128];
	swprintf (temp, L"[%d]", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const char*			msg)
{
	terse_		+= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const std::string&		msg)
{
	terse_		+= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const wchar_t*			msg)
{
	terse_		+= msg;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const std::wstring&	msg)
{
	terse_		+= msg;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(WStringCR				msg)
{
	terse_		+= msg.c_str ();
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(bool					state)
{
	terse_		+= state? std::wstring (L"true"): std::wstring (L"false");
}
///////////////////////////////////////////////////////////////////////
/*
void			CMessageTracer::Append	(int					number)
{
	WChar			temp	[128];
	swprintf (temp, L"%d", number);
	terse_		+= std::wstring (temp);
}
*/
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(UInt16					number)
{
	WChar			temp	[128];
	swprintf (temp, L"%u", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
/*
void			CMessageTracer::Append	(long					number)
{
	WChar			temp	[128];
	swprintf (temp, L"%ld", number);
	terse_		+= std::wstring (temp);
}
*/
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(UInt32					number)
{
	WChar			temp	[128];
	swprintf (temp, L"%u", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(Int32					number)
{
	WChar			temp	[128];
	swprintf (temp, L"%d", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
/*
void			CMessageTracer::Append	(std::size_t			number)
{
	WChar			temp	[128];
	swprintf (temp, L"%zu", number);
	terse_		+= std::wstring (temp);
}
*/
///////////////////////////////////////////////////////////////////////
//void			CMessageTracer::Append	(Bentley::ECN::ECValue const&	ecValue)
//{
//	Append (ecValue.GetKind ());
//}
///////////////////////////////////////////////////////////////////////
//void			CMessageTracer::Append	(Bentley::ECN::ValueKind	kind)
//{
//  switch (kind)
//  {
//  	case Bentley::ECN::ValueKind::VALUEKIND_Uninitialized:
//  		terse_	+=	L"ECValue is Uninitialized";
//  	break;
//  	case Bentley::ECN::ValueKind::VALUEKIND_Primitive:
//  		terse_	+=	L"ECValue stores a Primitive";
//  	break;
//  	case Bentley::ECN::ValueKind::VALUEKIND_Struct:
//  		terse_	+=	L"ECValue stores a Struct";
//  	break;
//  	case Bentley::ECN::ValueKind::VALUEKIND_Array:
//  		terse_	+=	L"ECValue stores an Array";
//  	break;
//  }
//}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::ElementId	(ElementHandleCR	eh)
{
	ElementId (eh.GetElementId ());
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::ElementId	(ElementRefP		er)
{
	ElementId (elementRef_getElemID (er));
}
///////////////////////////////////////////////////////////////////////
std::wstring	Format64bit (UInt64 			number)
{
	WChar			temp	[128];
	swprintf (temp, L"%I64u", number);
	return std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::ElementId	(Bentley::DgnPlatform::ElementId const id)
{
	terse_		+= Format64bit (id);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append		(UInt64 			number)
{
	terse_		+= Format64bit (number);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Degrees		(double				radians)
{
	WChar			temp	[128];
	swprintf (temp, L" %.0lf", radians * fc_180overpi);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Radians		(double				radians)
{
	WChar			temp	[128];
	swprintf (temp, L" %.4lf", radians);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Euler		(RotMatrix const&	r,
											int					precision)
{
	WChar			temp	[128];
	double			x, y, z, s;
	if (mdlRMatrix_isXRotationYRotationZRotationScale (&r, &x, &y, &z, &s))
		swprintf (temp, L" x='%.4lf' y='%.4lf' z='%.4lf' ", x, y, z);
	else
		swprintf (temp, L" matrix has non-uniform scale");
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::AppendDbl	(double				d,
											int					precision)
{
	WChar			format	[128];
	WChar			temp	[128];
	const WChar*	Format	(L"%%.%dlf");
	swprintf (format, Format, precision);
	swprintf (temp, format, d);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::AppendVolume(double				d,
											DgnModelRefP		modelRef,
											int					precision)
{
	const double	UORs	= nullptr == modelRef? 1.0: 1.0 / mdlModelRef_getUorPerMaster (modelRef);
	WChar			format	[128];
	WChar			temp	[128];
	const WChar*	Format	(L"%%.%dlf");
	swprintf (format, Format, precision);
	swprintf (temp, format, d * UORs * UORs * UORs);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::AppendArea	(double				d,
											DgnModelRefP		modelRef,
											int					precision)
{
	const double	UORs	= nullptr == modelRef? 1.0: 1.0 / mdlModelRef_getUorPerMaster (modelRef);
	WChar			format	[128];
	WChar			temp	[128];
	const WChar*	Format	(L"%%.%dlf");
	swprintf (format, Format, precision);
	swprintf (temp, format, d * UORs * UORs);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::AppendLength(double				d,
											DgnModelRefP		modelRef,
											int					precision)
{
	const double	UORs	= nullptr == modelRef? 1.0: 1.0 / mdlModelRef_getUorPerMaster (modelRef);
	WChar			format	[128];
	WChar			temp	[128];
	const WChar*	Format	(L"%%.%dlf");
	swprintf (format, Format, precision);
	swprintf (temp, format, d * UORs);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append		(RotMatrix const&	r,
											int					precision)
{
	WChar			format	[128];
	WChar			temp	[128];
	const WChar*	FormatRow	(L"\n| %%.%dlf, %%.%dlf, %%.%dlf |");
	swprintf (format, FormatRow, precision, precision, precision);
	swprintf (temp, format, r.form3d [0][0], r.form3d [0][1], r.form3d [0][2]);
	terse_		+= std::wstring (temp);
	swprintf (temp, format, r.form3d [1][0], r.form3d [1][1], r.form3d [1][2]);
	terse_		+= std::wstring (temp);
	swprintf (temp, format, r.form3d [2][0], r.form3d [2][1], r.form3d [2][2]);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Vector		(DVec3d const&		v,
											int					precision)
{
	WChar			format	[128];
	WChar			temp	[128];
	WChar const*	Format3D	(L"%%.%dlf,%%.%dlf,%%.%dlf");
	swprintf (format, Format3D, precision, precision, precision);
	swprintf (temp, format, v.x, v.y, v.z);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::IdentifyElm	(MSElementDescrCP	pElm)
{
	WChar			temp	[128];
	swprintf (temp, L" ID %I64d type %d", mdlElement_getID (&pElm->el), mdlElement_getType (&pElm->el));
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append		(DPoint3d const&	pt,
											DgnModelRefP		modelRef,
											int					precision)
{
	const double	UORs	= 1.0 / mdlModelRef_getUorPerMaster (modelRef);
	WChar			temp	[128];
	WChar			format	[128];
	if (mdlModelRef_is3D (modelRef))
	{
		const WChar*		Format3D	(L"x='%%.%dlf' y='%%.%dlf' z='%%.%dlf'");
		swprintf (format, Format3D, precision, precision, precision);
		swprintf (temp, format, pt.x * UORs, pt.y * UORs, pt.z * UORs);
	}
	else
	{
		const WChar*		Format2D	(L"x='%%.%dlf' y='%%.%dlf'");
		swprintf (format, Format2D, precision, precision);
		swprintf (temp, format, pt.x * UORs, pt.y * UORs);
	}
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::IsValid		(WCharCP				name,
											bool					valid)
{
	terse_	= name; terse_ += valid? L" is valid": L" is invalid";
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Message	(const char*		msg)
{
	terse_		= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Message	(const std::string&	msg)
{
	terse_		= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Verbose	(const char*			msg)
{
	verbose_	= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Verbose	(const std::string&		msg)
{
	verbose_	= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Verbose	(WCharCP				msg)
{
	verbose_	= msg;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Verbose	(const std::wstring&	msg)
{
	verbose_	= msg;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Verbose	(WStringCR				msg)
{
	verbose_	= msg.c_str ();
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const char*			msg)
:	terse_		(Convert (msg)),
	verbose_	(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const std::string&		msg)
:	terse_		(Convert (msg)),
	verbose_	(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const WChar*			msg)
:	terse_		(msg),
	verbose_	(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const std::wstring&	msg)
:	terse_		(msg),
	verbose_	(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(WStringCR				msg)
:	terse_		(msg.c_str ()),
	verbose_	(L"")
{
}
///////////////////////////////////////////////////////////////////////
