/*----------------------------------------------------------------------+
|																		|
| MessageTracer Copyright 2010..2017, LA Solutions Ltd					|
| All rights reserved.													|
|																		|
| Limited permission is hereby granted to reproduce and modify this		|
| copyrighted material provided that this notice is retained			|
| in its entirety in any such reproduction or modification.				|
|																		|
+----------------------------------------------------------------------*/
#if !defined(MESSAGE_TRACER_H_INCLUDED_)
#define MESSAGE_TRACER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
/*----------------------------------------------------------------------+
|                                                                       |
|   MessageTracer - MessageTracer source code. 							|
|                                                                       |
+----------------------------------------------------------------------*/
#include	<Mstn/MdlApi/msoutput.fdf>
#include	<DgnPlatform/DgnPlatformBaseType.r.h>
#include 	<string>

class CMessageTracer
{
	std::wstring	terse_;
	std::wstring	verbose_;

	static std::wstring	Convert			(std::string const&	s);
	static std::wstring	Convert			(char const* 		s);

public:
	///////////////////////////////////////////////////////////////////
	//	Implementation
	///////////////////////////////////////////////////////////////////
	///	<summary>	Send a message to MicroStation's message center.</summary>
	///	<remarks>	By default,
	///	the message is shown with DEBUG priority (user must turn on
	///	debug messages in the message center.</remarks>
	void			Output		(DgnPlatform::OutputMessagePriority	priority	= DgnPlatform::OutputMessagePriority::Debug)	const;
	void			Warning		(DgnPlatform::OutputMessagePriority	priority	= DgnPlatform::OutputMessagePriority::Warning,
								bool								pop			= true)		const;
	void			Advisory	(DgnPlatform::OutputMessagePriority	priority	= DgnPlatform::OutputMessagePriority::Warning,
								bool								pop			= false)	const
	{
		Warning (priority, pop);
	}
	///////////////////////////////////////////////////////////////////
	///	<summary>	Provide an alternative message to the current message.</summary>
	void			Message		(const char*			msg);
	void			Message		(const std::string&		msg);
	///////////////////////////////////////////////////////////////////
	///	<summary>	Provide a verbose message that differs from the terse message.</summary>
	void			Verbose		(const char*			msg);
	void			Verbose		(const std::string&		msg);
	void			Verbose		(const std::wstring&	msg);
	void			Verbose		(WStringCR				msg);
	void			Verbose		(WCharCP				msg);
	///////////////////////////////////////////////////////////////////
	///	<summary>	Append new text to the current message.</summary>
	void			NewLine		();
	void			Space		(int					count	= 1);
	///	<summary>	Wrap a string in quotes or brackets.</summary>
	enum BracketOption {	Parenthesis, Square, Brace, SingleQuote, DoubleQuote, };
	void			Bracket		(const char*			s, BracketOption bracket = Parenthesis);
	void			Bracket		(const std::string&		s, BracketOption bracket = Parenthesis);
	void			Bracket		(const WChar*			s, BracketOption bracket = Parenthesis);
	void			Bracket		(const std::wstring&	s, BracketOption bracket = Parenthesis);
	void			Bracket		(WStringCR				s, BracketOption bracket = Parenthesis);
	void			Quote		(const char*			suffix);
	void			Quote		(const std::string&		suffix);
	void			Quote		(const WChar*			suffix);
	void			Quote		(const std::wstring&	suffix);
	void			Quote		(WStringCR				suffix);
	std::string		Wrap		(const char*			s, const char* open, const char* close);
	std::string		Wrap		(const std::string&		s, const char* open, const char* close);
	std::wstring	Wrap		(const WCharCP			s, WCharCP open, WCharCP close);
	std::wstring	Wrap		(const std::wstring&	s, WCharCP open, WCharCP close);
	WString			Wrap		(WStringCR				s, WCharCP open, WCharCP close);
	void			Append		(const char*			suffix);
	void			Append		(const std::string&		suffix);
	void			Append		(const wchar_t*			suffix);
	void			Append		(const std::wstring&	suffix);
	void			Append		(WStringCR				uffix);
	void			Append		(bool					state);
	/*
	void			Append		(int					number);
	void			Append		(long					number);
	*/
	void			Append		(UInt16					number);
	void			Append		(UInt32					number);
	void			Append		(Int32					number);
	//void			Append		(Bentley::ECN::ECValue const&			ecValue);
	//void			Append		(Bentley::ECN::ValueKind 				kind);
	void			ElementId	(Bentley::DgnPlatform::ElementId const id);
	void			ElementId	(ElementHandleCR		eh);
	void			ElementId	(ElementRefP			er);
	///	<summary>UInt64 is Bentley::DgnPlatform::ElementId.</summary>
	void			Append		(UInt64 				number);
	//void			Append		(std::size_t			number);
	void			Append		(DPoint3d const&		pt,
								DgnModelRefP			modelRef	= ACTIVEMODEL,
								int						precision	= 2);
	///	<remarks>Vector method distinguishes DVec3d from DPoint3d</remarks>
	void			Vector		(DVec3d const&			v,
								int						precision	= 2);
	void			Append		(double					d,
								int						precision	= 2);
	void			AppendDbl	(double					d,
								int						precision	= 2);
	void			AppendArea	(double					d,
								DgnModelRefP			modelRef	= ACTIVEMODEL,
								int						precision	= 2);
	void			AppendLength(double					d,
								DgnModelRefP			modelRef	= ACTIVEMODEL,
								int						precision	= 2);
	void			AppendVolume(double					d,
								DgnModelRefP			modelRef	= ACTIVEMODEL,
								int						precision	= 2);
	void			Append		(double					d,
								DgnModelRefP			modelRef	= ACTIVEMODEL,
								int						precision	= 2);
	///	<remarks>Default precision of 7 matches Analyse Element command output</remarks>
	void			Append		(RotMatrix const&		r,
								int						precision	= 7);
	void			IdentifyElm	(MSElementDescrCP		el);
	void			Degrees		(double					radians);
	void			Radians		(double					radians);
	void			Euler		(RotMatrix const&		r,
								int						precision	= 7);
	void			Bracket		(int					number);
	void			IsValid		(WCharCP				name,
								bool					valid);
	///////////////////////////////////////////////////////////////////
	///	<summary>	Append operators.</summary>
	CMessageTracer&	operator =	(const char*				msg);
	CMessageTracer&	operator =	(const std::string&			msg);
	CMessageTracer&	operator =	(const WChar*				msg);
	CMessageTracer&	operator =	(const std::wstring&		msg);
	CMessageTracer&	operator =	(const Bentley::WString&	msg);
	CMessageTracer&	operator +=	(const char*				msg);
	CMessageTracer&	operator +=	(const std::string&			msg);
	CMessageTracer&	operator +=	(const WChar*				msg);
	CMessageTracer&	operator +=	(const std::wstring&		msg);
	CMessageTracer&	operator +=	(WStringCR					msg);
	CMessageTracer&	operator +=	(UInt32						number);
	CMessageTracer&	operator +=	(Int32						number);
	CMessageTracer&	operator +=	(bool						b);
	CMessageTracer&	operator +=	(double						d);
	///	<summary>	Index operator creates a string similar to '[123]'.</summary>
	///	<remarks>	Used for tracing the progress of for loops.</remarks>
	CMessageTracer&	operator []	(int						i);
	///////////////////////////////////////////////////////////////////
	//	Construction
	CMessageTracer	(const char*			msg);
	CMessageTracer	(const std::string&		msg);
	CMessageTracer	(const WChar*			msg);
	CMessageTracer	(const std::wstring&	msg);
	CMessageTracer	(WStringCR				msg);
};

#endif // !defined(MESSAGE_TRACER_H_INCLUDED_)
