#define DEBUG_VERSION 1

/*----------------------------------------------------------------------+
|									|
| Copyright , Bentley Systems, Incorporated, 			|
| All rights reserved.							|
|									|
|									|
| Limited permission is hereby granted to reproduce and modify this	|
| copyrighted material provided that the resulting code is used only in |
| conjunction with Bentley Systems products under the terms of the	|
| license agreement provided therein, and that this notice is retained	|
| in its entirety in any such reproduction or modification.		|
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|									|
|   $Workfile:   datasheet  $
|   $Revision:  $
|   	$Date:  $
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   datasheet - datasheet source code. |
|                                                                       |
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Include Files                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
#include <Mstn/MdlApi/mdl.h>
#include <Bentley/BeFileName.h>
#include <Mstn/cmdlist.r.h>
#include <Mstn/MdlApi/mscexpr.fdf>
#include <Mstn/MdlApi/dlogitem.h>
#include <Mstn/MdlApi/msoutput.fdf>
#include <Mstn/MdlApi/mdl.h>
#include <stdio.h>
#include <string.h>
#include <Mstn/MdlApi/msvar.fdf>
#include <Mstn/MdlApi/msdialog.fdf>
#include <Mstn/MdlApi/ditemlib.fdf>
#include <Mstn/MdlApi/msrmgr.h>
#include <Mstn/MdlApi/mssystem.fdf>
#include <Mstn/MdlApi/msparse.fdf>
#include <Mstn/MdlApi/listmodel.fdf>
#include <RmgrTools/Tools/toolsubs.h>
#include <Mstn/MdlApi/mswindow.fdf>
#include <Mstn/MdlApi/leveltable.fdf>
#include <Mstn/MdlApi/msmisc.fdf>
#include <Mstn/MdlApi/workmode.fdf>
#include <Mstn/MdlApi/msritem.fdf>
#include <Mstn/MdlApi/miscilib.fdf>

#include "CSVParser.h"
#include "datasheetCmd.h"
#include "datasheet.h"
#include "HookClasses.h"
#include "MessageTracer.h"

GlobalData  	gData;

///	<summary>Parse a Comma-Separated-Variable (CSV) text file.</summary>
int 		parseFile
(
BeFileName	fileName
)
{
    ListModelP	pListModel	= nullptr;
    MSDialogP	dbP = mdlDialog_find (DIALOGID_SpreadSheet, nullptr);
    if  (dbP)
	{
	    DialogItemP	diP = mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOXID_CellsSheet, 0);
	    pListModel = mdlDialog_listBoxGetListModelP (diP->rawItemP);
	}

	if (pListModel)
	{
		mdlListModel_empty (pListModel, true);
		///	<summary>Create and open a file buffer for input.</summary>
		std::wfilebuf		fb;
  		if (std::wfilebuf* pfb = fb.open (fileName.c_str (), std::ios::in))
		{
			///	<summary>Create input stream from the file buffer.</summary>
			std::wistream      	file (pfb);
			CSVRow              row;
			UInt32				nRow	= 0;
			while (file >> row)
			{
				///	<remarks>Fetch data from the input stream.</remarks>
				ListRowP      	pRow            { mdlListRow_create (pListModel) };
				const UInt32&	nListCols	= mdlListModel_getColumnCount (pListModel);
				const UInt32&	nDataCols	= (UInt32)row.size ();
				const UInt32&	Lesser		= nListCols < nDataCols? nListCols: nDataCols;
				for (UInt32	col = 0; col != Lesser; ++col)
				{
					///	<remarks>Copy data from the input stream into a ListModel.</remarks>
					ListCellP	pCell		= mdlListRow_getCellAtIndex (pRow, col);
					mdlListCell_setStringValue (pCell, row [col].c_str (), true);
				}
				mdlListModel_addRow (pListModel, pRow);
				++nRow;
			}
		}
		else
		{
			CMessageTracer moan ("Unable to open file"); moan.Quote (fileName.c_str ());
			moan.Advisory ();
		}
	}
    return SUCCESS;
}
/** --------------------------------------------------------------------+
|																		|
|	function name:	dataSheet_about										|
|																		|
|	description:	Pop 'About Datasheet' dialog						|
|																		|
|	author:			LA Solutions Ltd									|
|																		|
|	returns:		Nothing												|
|																		|
+--------------------------------------------------------------------- */
void 				dataSheet_about
(
WCharCP				unused
)
//cmdNumber 			CMD_DATASHEET_ABOUT
{
	CMessageTracer	msg (L"Datasheet example based on original by Bentley Systems, Inc. with additional material from LA Solutions Ltd");
	msg.Verbose (L"LA Solutions Ltd http://www.la-solutions.co.uk");
	msg.Output (DgnPlatform::OutputMessagePriority::Info);
}
/*----------------------------------------------------------------------+*//**
* !!!Describe Function Completely!!!								    	    *
* @bsimethod dataSheet_openCsvFile   					*					    	*
* @param 	nameOfParam     !!!what this parameter means	    	*
* @return 	!!!what the return value means			    	*
*								        *
* Author:   MarkAnderson 				04/01 		*
*								        *
* Handles:	CMD_DATASHEET_FILE_CSV_OPEN
+----------------------------------------------------------------------*/
DLLEXPORT  void dataSheet_openCsvFile
(
WCharCP	    	candidate
)
{
	const bool	HaveFile	{ !WString::IsNullOrEmpty (candidate) && BeFileName::DoesPathExist (candidate)};
	if (HaveFile)
	{
		//	Open passed file
		parseFile (BeFileName (candidate));
	}
	else
	{
		//	prompt user for save file name
		BeFileName    fileName;
		if (!mdlDialog_fileOpen (fileName, 0, 0, nullptr, L"*.csv", nullptr, L"Select CSV File"))
			parseFile (fileName);
	}

    return;
}
///////////////////////////////////////////////////////////////////////
///	<summary>Save the contents of the DataSheet to a CSV file.</summary>
///	<remarks>Handles CMD_DATASHEET_FILE_CSV_SAVEAS.</remarks>
DLLEXPORT  void datasheet_saveCsvFileAs
(
WCharCP	    	candidate		//	=>	May be NULL or empty
)
{
	const bool	HaveFile	{ !WString::IsNullOrEmpty (candidate) && BeFileName::DoesPathExist (candidate)};
	if (HaveFile)
	{
		//	Save passed file
	}
	else
	{
		//	prompt user for save file name
		BeFileName	fileName;
		switch (mdlDialog_fileCreate (fileName, 0, 0, L"DataSheet.csv", L"*.csv", L"MS_MDLAPPS", L"Save CSV File As..."))
		{
			case true:
				//	Cancel
			break;
			case false:
				//	OK
			break;
			case ERROR:
				//	Problem
			break;
		}
	}
}
/*----------------------------------------------------------------------+*//**
* A function to open the dialog								    	    *
* @bsimethod dataSheet_openDialog   					*					    	*
* @param 	unparsed     extra parameters sent to the command.	    	*
*								        *
* Author:   MarkAnderson 				04/01 		*
*								        *
+----------------------------------------------------------------------*/
DLLEXPORT  void dataSheet_openDialog
(
WCharCP		unparsed
) //cmdNumber CMD_DATASHEET_OPENDIALOG
{
    MSDialogP	dbP	= mdlDialog_find (DIALOGID_SpreadSheet, 0);
    if  (!dbP)
    	dbP = mdlDialog_open (0, DIALOGID_SpreadSheet);
}
///	<summary>Unload this application.</summary>
DLLEXPORT	void	dataSheet_exit
(
WCharCP		unparsed
)
{
	const bool&		LocalCommand	= false;
	mdlDialog_cmdNumberQueue (LocalCommand, CMD_MDL_UNLOAD, mdlSystem_getCurrTaskID (), MSInputQueuePos::INPUTQ_EOQ);
}
/*---------------------------------------------------------------------------------**//**
* @description  datasheet_unimplementedCommand
* @param 	unparsed      The unparsed information sent to the command
* @bsimethod 							BSI             06/03
+---------------+---------------+---------------+---------------+---------------+------*/
extern "C" DLLEXPORT void datasheet_unimplementedCommand
(
WCharCP unparsed
)
{
	CMessageTracer	msg (L"Command Not Implemented");
	msg.Advisory ();
}
/*---------------------------------------------------------------------------------**//**
* @description  MdlMain
* @param 	argc      The number of command line parameters sent to the application.
* @param 	argv[]    The array of strings sent to the application on the command line.
* @bsimethod 							BSI             06/03
+---------------+---------------+---------------+---------------+---------------+------*/
extern "C" DLLEXPORT void MdlMain (int argc, WCharCP argv[])
{
	RscFileHandle   rfHandle;
    mdlResource_openFile (&rfHandle,nullptr,RSC_READONLY);

    gData.colorNumber = 0;

    DialogHookInfo uHooks[]=
    {
    {HOOKITEMID_ListBox,  			(PFDialogHook) DataListBoxHandler::HookResolve			},
    {HOOKITEMID_ComboBoxText, 		(PFDialogHook) ComboBoxCell_TextEditor::HookResolve		},
    {HOOKITEMID_ComboBoxYesNo,		(PFDialogHook) ComboBoxCell_BoolEditor::HookResolve		},
    {HOOKITEMID_ComboBoxPerRow,		(PFDialogHook) ComboBoxCell_PerRow::HookResolve			},
	{HOOKITEMID_ListCellTextEditor,	(PFDialogHook) ListCellTextEditor::HookResolve			},
    {HOOKITEMID_ComboBoxLevels,		(PFDialogHook) ComboBoxCell_LevelEditor::HookResolve	},
    {HOOKITEMID_ToggleButton,		(PFDialogHook) ToggleButtonCell_BoolEditor::HookResolve	},
    {HOOKITEMID_TextColor,			(PFDialogHook) ListCellColorEditor::HookResolve			},
    {HOOKITEMID_ColorPicker,		(PFDialogHook) ColorPickerCell_ColorEditor::HookResolve	},
	{HOOKITEMID_ColorChooser,	    (PFDialogHook) ColorChooserCell_ColorEditor::HookResolve},
    };

    mdlDialog_hookPublish (sizeof (uHooks)/sizeof (uHooks[0]), uHooks);

    SymbolSet*	setP = mdlCExpression_initializeSet (VISIBILITY_DIALOG_BOX, 0, true);
    mdlDialog_publishComplexVariable (setP, "globaldata", "gData", &gData);

     // Map command name to function (usage: MDL COMMAND COMPEXPORT)
    static  MdlCommandName cmdNames[] =
    {
        {datasheet_unimplementedCommand, "datasheet_unimplementedCommand"  },
        0,
    };

    mdlSystem_registerCommandNames (cmdNames);

    // Map key-in to function
    static MdlCommandNumber cmdNumbers[] =
    {
		{dataSheet_openCsvFile,  			CMD_DATASHEET_FILE_CSV_OPEN 	},
        {datasheet_saveCsvFileAs,			CMD_DATASHEET_FILE_CSV_SAVEAS 	},
        {dataSheet_openDialog, 	 			CMD_DATASHEET_DIALOG_OPEN 		},
        {datasheet_unimplementedCommand,	CMD_DATASHEET_DIALOG_CLOSE 		},
        {dataSheet_about,					CMD_DATASHEET_HELP_ABOUT 		},
        {dataSheet_exit,					CMD_DATASHEET_FILE_EXIT 		},
        0,
    };

    mdlSystem_registerCommandNumbers (cmdNumbers);

    mdlParse_loadCommandTable (nullptr);

    dataSheet_openDialog (nullptr);
}
